<?php

defined( 'ABSPATH' ) || exit;

/**
 * Class SYSP_GM
 *
 * ver: 1.02
 *
 */
class SYSP_GM {
	protected $version;

	public function __construct() {
		$this->version = SYNCSPIDER_VERSION;

		// syncspider menu
		add_action( 'admin_init', array( $this, 'settings' ), 20 );
		add_filter( 'syncspider_menu_items', array( $this, 'admin_menu' ), 10, 1 );
		add_filter( 'woocommerce_rest_prepare_shop_order_object', array( $this, 'rest_invoice' ), 10, 3 );

        add_filter( 'syncspider_add_order_meta', array( $this, 'order_meta'), 10, 1);
	}

	public function settings() {

		if ( ! wp_doing_ajax() ) {
			add_settings_section(
				'syncspider_gm_settings',
				'',
				'',
				'syncspider_gm_settings'
			);

			register_setting(
				'syncspider_gm_fields',
				'syncspider_gm_settings',
				array( 'sanitize_callback' => array( $this, 'sanitize_gm_settings' ) )
			);

			add_settings_field(
				'syncspider_all_gm_settings',
				'',
				array( $this, 'print_fields' ),
				'syncspider_gm_settings',
				'syncspider_gm_settings'
			);

		}
	}

	/**
	 * @param $settings
	 *
	 * @return mixed
	 */
	public function sanitize_gm_settings( $settings ) {

		foreach ( $settings as $key => &$set ) {

			if ( '0' === $set ) {
				unset( $settings[ $key ] );
			} else {
				$set = 1;
			}
		}

		return $settings;
	}

	public function admin_menu( $menu_items ) {
		$menu_items[1300] = array(
			'title'    => __( 'German Market', 'syncspider' ),
			'slug'     => 'gm',
			'callback' => array( $this, 'admin_settings' ),
		);

		return $menu_items;
	}


	public function admin_settings() {
		?>
        <form method="post" action="options.php">
			<?php
			settings_fields( 'syncspider_gm_fields' );
			do_settings_sections( 'syncspider_gm_settings' );
			submit_button();
			?>
        </form>
		<?php
	}


	public function print_fields() {

		$settings = $this->get_syncspider_gm_settings();

		?>
        <br>
        <h3><?php _e( 'German Market support options', 'syncspider' ); ?></h3>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_gm_pdf_invoice_link" name="syncspider_gm_settings[pdf_invoice_link]"
                       value="1" <?php checked( true, isset( $settings['pdf_invoice_link'] ), true ); ?> autocomplete="off"/>
				<?php _e( 'Activate pdf invoice link in order meta data', 'syncspider' ); ?>
            </label>
        </fieldset>

        <fieldset id="syncspider_gm_pdf_invoice_link_sub" class="syncspider-subfield <?php echo( isset( $settings['pdf_invoice_link'] ) ? "" : "syncspider-hide" ); ?>">
            <div>
                <label>
                    <input type="checkbox" id="syncspider_gm_pdf_invoice_generate"
                           name="syncspider_gm_settings[pdf_invoice_generate]"
                           value="1" <?php checked( true, isset( $settings['pdf_invoice_generate'] ), true ); ?> autocomplete="off"/>
					<?php _e( 'Show download link even if invoice is not yet generated', 'syncspider' ); ?>
                    <p><em>Once download link is visited invoice will be generated.</em></p>
            </div>
        </fieldset>

        <script>
            jQuery(document).ready(function ($) {
                $('#syncspider_gm_pdf_invoice_link').on('click', function () {
                    setTimeout(function () {
                        if ($('#syncspider_gm_pdf_invoice_link').prop('checked') == true) {
                            $('#syncspider_gm_pdf_invoice_link_sub').slideDown(200);
                        } else {
                            $('#syncspider_gm_pdf_invoice_link_sub').slideUp(200);
                        }
                    }, 100);
                });
            });
        </script>
		<?php
	}

	/**
     * Register additional meta so Syncspider can add it to schema
	 * @param $meta
	 *
	 * @return mixed
	 */
	public function order_meta( $meta ) {
        $settings = $this->get_syncspider_gm_settings();
        if(isset($settings['pdf_invoice_link'])) {
            $meta[] = array(
                'slug'  => 'syncspider_gm_pdf_invoice_url',
                'label' => 'German Market pdf invoice url',
            );
        }

		return $meta;
	}

	public function rest_invoice( $response, $order, $request ) {
		$invoice_url = $this->get_invoice_download_url( $order );

		if ( ! isset( $response->data['meta_data'] ) ) {
			$response->data['meta_data'] = array();
		}

		// omitting id, as this is only used by syncspider todo: ask @ladislav
		$response->data['meta_data'][] = array(
			'key'   => 'syncspider_gm_pdf_invoice_url',
			'value' => $invoice_url,
		);

		return $response;
	}


	public function get_invoice_download_url( WC_Order $order ) {

		$invoice_url = '';

		if ( is_object( $order ) && class_exists( 'WP_WC_Invoice_Pdf_Email_Attachment' ) ) {

			$show_pdf_anyway = $this->get_syncspider_gm_settings()['pdf_invoice_generate'] ?? false;

			if ( $show_pdf_anyway || $this->invoice_exists( $order ) ) {
				$security_length = 12;
				$security_salt   = 'woocommerce-german-market';

				$email_address     = $order->get_billing_email();
				$email_hash        = function_exists( 'hash' ) ? hash( 'sha256', $email_address ) : sha1( $email_address );
				$shop_url          = esc_url( wc_get_page_permalink( 'shop' ) );
				$invoice_order_id  = $order->get_id();
				$invoice_order_key = $order->get_order_key();
				$invoice_url       = $shop_url . '?gm_invoice_pdf_order_id=' . $invoice_order_id . '&order_key=' . $invoice_order_key . '&order_hash=' . substr( md5( $invoice_order_id . '-' . $invoice_order_key . '-' . $security_salt ), 0, $security_length ) . '&email=' . $email_hash;
			}
		}

		return $invoice_url;
	}

	/**
	 * Check if invoice is already generated
	 *
	 * @param $order
	 *
	 * @return bool
	 */
	private function invoice_exists( $order ) {
		$exists = false;

		if ( class_exists( 'WP_WC_Invoice_Pdf_Order_Meta' ) ) {
			$invoice_pdf_order_meta = new WP_WC_Invoice_Pdf_Order_Meta( $order );
			$exists                 = $invoice_pdf_order_meta->has_meta();
		}

		return $exists;
	}

	private function get_syncspider_gm_settings() {

		$settings = get_option( 'syncspider_gm_settings' );

		return $settings;
	}

}

new SYSP_GM();