<?php

defined( 'ABSPATH' ) || exit;

/**
 * Class SYSP_Mailpoet
 */
class SYSP_Mailpoet {
	protected $version;
	private $mailpoet_api;

	public function __construct() {
		$this->version = SYNCSPIDER_VERSION;


		// syncspider menu
		add_action( 'admin_init', array( $this, 'settings' ), 20 );
		add_filter( 'syncspider_menu_items', array( $this, 'admin_menu' ), 10, 1 );
		if ( class_exists( \MailPoet\API\API::class ) ) {
			$this->mailpoet_api = \MailPoet\API\API::MP( 'v1' );

			add_action( 'rest_api_init', array( $this, 'register_routes' ) );

		}

	}

	/**
	 * Register all mailpoet routes
	 * @link https://github.com/mailpoet/mailpoet/blob/trunk/doc/Readme.md
	 * @date 10.11.2023.
	 */
	public function register_routes() {


		register_rest_route( 'wp/v2/syncspider', 'mailpoet/isSetupComplete', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'isSetupComplete' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/getLists', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'getLists' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/getSubscriber', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'getSubscriber' ),
				'permission_callback' => array( $this, 'permission_check' ),
				'args'                => array(
					'email' => array(
						'type'              => 'string',
						'required'          => true,
						'validate_callback' => 'is_email',
					),
				),
			)
		);
		register_rest_route( 'wp/v2/syncspider', 'mailpoet/getSubscribers', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'getSubscribers' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/getSubscribersCount', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'getSubscribersCount' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/getSubscriberFields', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'getSubscriberFields' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/addSubscriber', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'addSubscriber' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/addSubscriberField', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'addSubscriberField' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/addList', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'addList' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		/**
		 * NOTE: You cannot update only description.
		 */
		register_rest_route( 'wp/v2/syncspider', 'mailpoet/updateList', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'updateList' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/deleteList', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'deleteList' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/unsubscribe', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'unsubscribe' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/unsubscribeFromList', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'unsubscribeFromList' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/unsubscribeFromLists', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'unsubscribeFromLists' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/subscribeToList', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'subscribeToList' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

		register_rest_route( 'wp/v2/syncspider', 'mailpoet/subscribeToLists', array(
				'methods'             => 'POST',
				'callback'            => array( $this, 'subscribeToLists' ),
				'permission_callback' => array( $this, 'permission_check' ),
			)
		);

	}

	/**
	 * @return WP_REST_Response
	 */
	public function isSetupComplete() {
		return new WP_REST_Response( $this->mailpoet_api->isSetupComplete() );
	}

	/**
	 * @return WP_REST_Response
	 */
	public function getLists() {
		$lists = $this->mailpoet_api->getLists();

		return new WP_REST_Response( $lists );
	}

	/**
	 * @param $request
	 *
	 * @return WP_REST_Response
	 */
	public function getSubscriber( $request ) {
		$params = $request->get_params();
		try {
			$user = $this->mailpoet_api->getSubscriber( $params['email'] );

			return new WP_REST_Response( $user, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	/**
	 * todo: check with devs about timestam!
	 * see minUpdatedAt @link https://github.com/mailpoet/mailpoet/blob/trunk/doc/api_methods/GetSubscribers.md
	 * @return WP_REST_Response
	 */
	public function getSubscribers( $request ) {
		$params = $request->get_params();

		$filter = array();
		$limit  = 50;
		$offset = 0;

		if ( isset( $params['filter'] ) && is_array( $params['filter'] ) ) {

			if ( ! empty( $params['filter']['status'] ) ) {
				$filter['status'] = sanitize_text_field( $params['filter']['status'] );
			}
			if ( ! empty( $params['filter']['listId'] ) && is_numeric( $params['filter']['listId'] ) ) {
				$filter['listId'] = (int) $params['filter']['listId'];
			}
			if ( ! empty( $params['filter']['minUpdatedAt'] ) && is_numeric( $params['filter']['minUpdatedAt'] ) ) {
				$filter['minUpdatedAt'] = (int) $params['filter']['minUpdatedAt'];
			}
		}

		if ( isset( $params['limit'] ) ) {
			$limit = (int) $params['limit'];
		}

		if ( isset( $params['offset'] ) ) {
			$offset = (int) $params['offset'];
		}

		return new WP_REST_Response( $this->mailpoet_api->getSubscribers( $filter, $limit, $offset ) );
	}

	/**
	 * @return WP_REST_Response
	 */
	public function getSubscribersCount( $request ) {
		$params = $request->get_params();

		$filter = array();

		if ( isset( $params['filter'] ) && is_array( $params['filter'] ) ) {

			if ( ! empty( $params['filter']['status'] ) ) {
				$filter['status'] = sanitize_text_field( $params['filter']['status'] );
			}
			if ( ! empty( $params['filter']['listId'] ) && is_numeric( $params['filter']['listId'] ) ) {
				$filter['listId'] = (int) $params['filter']['listId'];
			}
			if ( ! empty( $params['filter']['minUpdatedAt'] ) && is_numeric( $params['filter']['minUpdatedAt'] ) ) {
				$filter['minUpdatedAt'] = (int) $params['filter']['minUpdatedAt'];
			}
		}

		return new WP_REST_Response( $this->mailpoet_api->getSubscribersCount( $filter ) );
	}

	/**
	 * @return array
	 * @throws Exception
	 */
	public function getSubscriberFields() {
		return $this->mailpoet_api->getSubscriberFields();
	}


	public function addSubscriber( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$sub = $body['subscriber'] ?? array();

		$subscriber = array(
			'email' => $sub['email'] ?? "",
		);

		if ( ! empty( $sub['first_name'] ) ) {
			$subscriber['first_name'] = $sub['first_name'];
		}

		if ( ! empty( $sub['last_name'] ) ) {
			$subscriber['last_name'] = $sub['last_name'];
		}

		foreach ( $sub as $sub_key => $value ) {
			if ( strpos( $sub_key, 'cf_' ) === 0 ) {
				$subscriber[ $sub_key ] = $value;
			}
		}

		if ( ! empty( $body['list_ids'] ) && is_array( $body['list_ids'] ) ) {
			$list_ids = $body['list_ids'];
		} else {
			$list_ids = array();
		}

		if ( ! empty( $body['options'] ) && is_array( $body['options'] ) ) {
			$options = $body['options'];
		} else {
			$options = array();
		}

		try {
			$user = $this->mailpoet_api->addSubscriber( $subscriber, $list_ids, $options );

			return new WP_REST_Response( $user, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function addSubscriberField( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$data = $body['data'] ?? array();

		try {
			$response = $this->mailpoet_api->addSubscriberField( $data );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function addList( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$list = $body['list'] ?? array();

		try {
			$response = $this->mailpoet_api->addList( $list );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}


	public function updateList( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$list = $body['list'] ?? array();

		try {
			$response = $this->mailpoet_api->updateList( $list );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function deleteList( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$list_id = (string) $body['list_id'] ?? "";

		try {
			$response = $this->mailpoet_api->deleteList( $list_id );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function unsubscribe( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$subscriber_id = (string) ($body['subscriber_id'] ?? "");

		try {
			$response = $this->mailpoet_api->unsubscribe( $subscriber_id );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function unsubscribeFromList( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$subscriber_id = (string) ($body['subscriber_id'] ?? "");
		$list_id = (string) ($body['list_id'] ?? "");

		try {
			$response = $this->mailpoet_api->unsubscribeFromList( $subscriber_id, $list_id );

			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function unsubscribeFromLists( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$subscriber_id = (string) ($body['subscriber_id'] ?? "");
		$list_ids = $body['list_ids'] ?? array();

		try {
			$response = $this->mailpoet_api->unsubscribeFromLists( $subscriber_id, $list_ids );
			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function subscribeToList( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$subscriber_id = (string) ($body['subscriber_id'] ?? "");
		$list_id = (string) $body['list_id'] ?? "";
        $options = $body['options'] ?? array();

		try {
			$response = $this->mailpoet_api->subscribeToList( $subscriber_id, $list_id, $options  );
			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}

	public function subscribeToLists( WP_REST_Request $request ) {
		$body = $request->get_json_params();

		$subscriber_id = (string) ($body['subscriber_id'] ?? "");
		$list_ids = $body['list_ids'] ?? array();
        $options = $body['options'] ?? array();

		try {
			$response = $this->mailpoet_api->subscribeToLists( $subscriber_id, $list_ids, $options );
			return new WP_REST_Response( $response, 200 );
		} catch ( Exception $e ) {
			return $this->get_error( $e );
		}
	}


	/**
	 * Get response for error, status 400
	 *
	 * @param $e Exception
	 *
	 * @return WP_REST_Response
	 */
	public function get_error( $e ) {
		$error = array(
			'code'    => $e->getCode(),
			'message' => $e->getMessage(),
			'data'    => array(
				'status' => 400,
			),
		);

		return new WP_REST_Response( $error, 400 );
	}


	/**
	 * Check permissions. Our routes are not public so this might not be needed.
	 *
	 * @return bool
	 */
	public function permission_check() {
		if ( SYSP_check::check_rest_view() ) {
			return true;
		}

		return false;
	}


	public function settings() {

		if ( ! wp_doing_ajax() ) {
			add_settings_section(
				'syncspider_mailpoet_settings',
				'',
				'',
				'syncspider_mailpoet_settings'
			);

			register_setting(
				'syncspider_mailpoet_fields',
				'syncspider_mailpoet_settings',
				array( 'sanitize_callback' => array( $this, 'sanitize_mailpoet_settings' ) )
			);

			add_settings_field(
				'syncspider_all_mailpoet_settings',
				'',
				array( $this, 'print_fields' ),
				'syncspider_mailpoet_settings',
				'syncspider_mailpoet_settings'
			);

		}
	}


	public function print_fields() {
		
	}


	/**
	 * @param $settings
	 *
	 * @return mixed
	 */
	public function sanitize_mailpoet_settings( $settings ) {
		foreach ( $settings as $key => &$set ) {
			if ( '0' === $set ) {
				unset( $settings[ $key ] );
			} else {
				$set = 1;
			}
		}

		return $settings;
	}

	public function admin_menu( $menu_items ) {
		$menu_items[1300] = array(
			'title'    => 'Mailpoet',
			'slug'     => 'mailpoet',
			'callback' => array( $this, 'admin_settings' ),
		);

		return $menu_items;
	}

	public function admin_settings() {
		?>
        <form method="post" action="options.php">
			<?php
			settings_fields( 'syncspider_mailpoet_fields' );
			do_settings_sections( 'syncspider_mailpoet_settings' );
			submit_button();
			?>
        </form>
		<?php
	}

}

new SYSP_Mailpoet();