<?php

/**
 * SyncSpider Admin
 *
 *
 * @class    SYSP_Admin
 * @version  1.0.3
 */

defined( 'ABSPATH' ) || exit;

class SYSP_Admin {

	protected $version;

	public function __construct( $version ) {
		$this->version = $version;
		add_action( 'admin_menu', array( $this, 'syncspider_backend' ), 5 );

		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ), 15 );
		add_action( 'wp_ajax_syncspider_update_addons', array( $this, 'save_addons_settings' ) );

		add_action( 'wp_ajax_syncspider_generate_key', array( $this, 'syncspider_generate_key' ) );
	}

	/**
	 * @return mixed
	 */
	public function getVersion() {
		return $this->version;
	}

	// create custom plugin menu in left sidebar
	public function syncspider_backend() {
		add_menu_page(
			'Syncspider',
			'Syncspider',
			'activate_plugins',
			'syncspider-settings',
			array( $this, 'settings_page' ),
			plugins_url( 'assets/images/icon.png', SYNCSPIDER_PLUGIN_FILE ) );
	}

	// add scripts only on syncspider settings page
	public function enqueue_admin_scripts( $hook ) {
		if ( 'toplevel_page_syncspider-settings' === $hook ) {
			wp_enqueue_style( 'syncspider_admin', plugins_url( 'assets/css/admin.css', SYNCSPIDER_PLUGIN_FILE ), array(), $this->getVersion() );
			wp_enqueue_script( 'syncspider_admin', plugins_url( 'assets/js/admin.js', SYNCSPIDER_PLUGIN_FILE ), array(), $this->getVersion(), true );
		}
	}


	public function settings_page() {

		$menu_items = $this->get_menu_items();

		$admin_url    = get_admin_url() . 'admin.php?page=syncspider-settings';
		$current_item = current( $menu_items );
		?>
        <div class="wrap">
            <div class="syncspider-wrap">
                <div class="syncspider-menu">
                    <div class="syncspider-logo"></div>

                    <div class="syncspider-menu">
                        <ul>
							<?php
							foreach ( $menu_items as $item ) {
								$class = '';
								if ( isset ( $_GET['tab'] ) ) {
									if ( $_GET['tab'] === $item['slug'] ) {
										$class        = 'current-item';
										$current_item = $item;
									}
								}
								echo '<li class="' . $class . '"><a href="' . $admin_url . '&tab=' . $item['slug'] . '">' . $item['title'] . '</a></li>';
							}
							?>
                        </ul>
                    </div>
                    <div class="syncspider-version">
                        <small><?php _e( 'Version:', 'syncspider' ); ?><?php echo $this->getVersion(); ?></small>
                    </div>

                </div>
                <div class="syncspider-content">
                    <div>
						<?php $this->show_current_content( $current_item ); ?>
                    </div>
                </div>
            </div>
        </div>
		<?php
	}

	/**
	 * Get menu items
	 *
	 * @access private
	 * @return array
	 */
	private function get_menu_items() {
		$menu_items = array(
			0 => array(
				'title'    => __( 'General', 'syncspider' ),
				'slug'     => 'general',
				'callback' => array( $this, 'settings_general' ),
			),
		);

		if ( SYSP_check::all_addons() ) {
			$menu_items[600] = array(
				'title'    => __( 'Add-ons', 'syncspider' ),
				'slug'     => 'addons',
				'callback' => array( $this, 'settings_addons' ),
			);
		}

		// add additional items based on addons settings
		$menu_items = apply_filters( 'syncspider_menu_items', $menu_items );

		ksort( $menu_items );

		return $menu_items;
	}

	/**
	 * Display current tab content
	 *
	 * @param $current_item
	 */
	private function show_current_content( $current_item ) {

		$admin_url = get_admin_url() . 'admin.php?page=syncspider-settings&tab=' . $current_item['slug'];
		?>
        <div class="syncspider_show_nag"><small>Show wp messages ></small></div>
        <div style="display: none;" class="syncspider_nag">
            <h2 style="display: none;">Hide nag</h2>
        </div>

        <h2><?= $current_item['title']; ?></h2>

		<?php
		if ( ! empty( $current_item['submenu'] ) ) { ?>
            <div class="syncspider-submenu nav-tab-wrapper">
				<?php
				$submenu             = $current_item['submenu'];
				$current_submenu_arg = $_GET['submenu'] ?? '';

				if ( isset( $submenu[ $current_submenu_arg ] ) ) {
					$current_submenu = $current_submenu_arg;
				} else {
					$current_submenu = key( $submenu );
				}

				foreach ( $submenu as $param => $menu ) {
					$url = esc_url( $admin_url . '&submenu=' . $param );
					echo '<a class="nav-tab syncspider-submenu-item ' . ( $current_submenu === $param ? 'current-submenu nav-tab-active' : '' ) . '" href="' . $url . '">' . $menu['name'] . '</a>';
				}
				?>
            </div>
			<?php
			if ( is_callable( $submenu[ $current_submenu ]['callback'] ) ) {
				call_user_func( $submenu[ $current_submenu ]['callback'] );
			}
		} else {
			if ( is_callable( $current_item['callback'] ) ) {
				call_user_func( $current_item['callback'] );
			}
		}
	}

	/**
	 * General settings. Generate api key
	 */
	private function settings_general() {
		if ( current_user_can( 'activate_plugins' ) ) {
			syncspider_register_plugin_settings();

			$api_key_string = get_option( 'wp_syncspider_api_string' );
			$api_key_hash   = get_option( 'wp_syncspider_api_hash' );
			$api_key_user   = get_option( 'wp_syncspider_user_id' );

			$user = get_user_by( 'id', (int) $api_key_user );
			wp_nonce_field( 'syncspider_general_settings_nonce', 'syncspider_general_settings_sc' );

			if ( $api_key_string && $api_key_hash && $api_key_user ) {

				echo '<p  id="syncspider-general__key_string">' . __( 'Your api key starts with: ', 'syncspider' ) . ' <span id="syncspider-general__key">' . esc_html( $api_key_string ) . '... </span></p>';
				?>
                <div style="display: none;" id="syncspider-general__generate">
                    <div class="button">Generate new</div>
                    <div id="syncspider-general__generate_key" style="display: none; border: 1px solid #ddd; padding: 10px;">
                        <div><strong>Generate new api key</strong></div>
                        <p>
                            <label>
                                <input id="syncspider-general__generate_manual" type="checkbox">
                                Enter key manually
                            </label>
                        </p>
                        <div id="syncspider-general__generate_manual_key" style="padding-left: 24px; display: none;">
                            <label>
                                <p>Key must contain at least 20 characters</p>
                                <input class="" minlength="20" placeholder="key..." maxlength="50" type="text">
                            </label>
                        </div>

                        <div style="margin-bottom: 20px;">
                            <label><p>Old key will be deleted. <br>Please confirm by typing 'Generate' in following field.</p>
                                <input id="syncspider-general__generate_confirm" placeholder="Generate" type="text">
                            </label>
                        </div>

                        <div id="syncspider-general__generate-message" style="margin-bottom: 15px; color: red; display: none;"></div>

                        <div id="syncspider-general__generate-confirm" class="syncspider-button__save"
                             data-strsaved="<?php _e( 'Changes saved', 'syncspider' ) ?>"
                             data-strsave="<?php _e( 'Generate new', 'syncspider' ) ?>"
                             data-strsaving="<?php _e( 'Saving changes', 'syncspider' ) ?>"><?php _e( 'Generate new', 'syncspider'
							) ?>
                        </div>
                        <div id="syncspider-general__generate_cancel" class="button">Cancel</div>
                    </div>
                </div>


				<?php
				if ( $user ) {
					echo '<p>' . __( 'User associated with key: ', 'syncspider' ) . ' <span><a href="' . get_edit_user_link( $api_key_user ) . '">' . $user->nickname . '</a></span></p>';
					if ( ! user_can( $user, 'activate_plugins' ) ) {
						echo '<p style="color: red;">' . __( 'Seems like associated user does not have required privileges, please generate API key again!', 'syncspider' ) . '</p>';
					}
				} else {
					echo '<p>' . __( 'User associated with key does not exist', 'syncspider' ) . ' (id: ' . $api_key_user . '). ' . __( 'Please generate your key again.', 'syncspider' ) . ' </p>';
				}
			} else { ?>
                <div id="syncspider-general__generate">
                    <div id="syncspider-general__generate_key">
                        <div style="margin-bottom: 15px;"><strong>Generate new api key</strong></div>

                        <div id="syncspider-general__generate-message" style="margin-bottom: 15px; color: red; display: none;"></div>
                        <input id="syncspider-general__generate_confirm" value="Generate" type="hidden">
                        <input id="syncspider-general__generate_manual_key" value="Generate" type="hidden">
                        <div id="syncspider-general__generate-confirm" class="syncspider-button__save"
                             data-strsaved="<?php _e( 'Changes saved', 'syncspider' ) ?>"
                             data-strsave="<?php _e( 'Generate new', 'syncspider' ) ?>"
                             data-strsaving="<?php _e( 'Saving changes', 'syncspider' ) ?>"><?php _e( 'Generate new', 'syncspider'
							) ?>
                        </div>
                    </div>
                </div>
				<?php
			}
		}
	}


	/**
	 * Ajax callback for key generation
	 */
	public function syncspider_generate_key() {
		check_ajax_referer( 'syncspider_general_settings_nonce', 'sysp_um' );

		if ( current_user_can( 'activate_plugins' ) && isset( $_POST['manual'] ) && isset( $_POST['manual_key'] ) && isset( $_POST['generate'] ) ) {
			$password = $hash = '';

			if ( $_POST['generate'] !== 'Generate' ) {
				$msg = __( 'Please confirm.', 'syncspider' );
				wp_send_json_error( $msg );
			}

			if ( $_POST['manual'] === 'true' ) {
				if ( empty( $_POST['manual_key'] ) ) {
					$msg = __( 'Please enter valid key.', 'syncspider' );
					wp_send_json_error( $msg );
				}
				$length = mb_strlen( $_POST['manual_key'] );

				if ( $length < 20 || $length > 40 || preg_match( '/\s|<|>/', $_POST['manual_key'] ) ) {
					$msg = __( 'Please enter valid key.', 'syncspider' );
					wp_send_json_error( $msg );
				}
				$password = $_POST['manual_key'];
				$hash     = wp_hash_password( $_POST['manual_key'] );
			}

			if ( ! $hash ) {
				$password = wp_generate_password( 20 );
				$hash     = wp_hash_password( $password );
			}

			update_option( 'wp_syncspider_api_hash', $hash, true );
			update_option( 'wp_syncspider_api_hash', $hash, true );
			update_option( 'wp_syncspider_user_id', get_current_user_id(), true );
			update_option( 'wp_syncspider_api_string', mb_substr( $password, 0, 5 ), true );
			$message = __( 'Your api key is:', 'syncspider' );
			$message .= '<br><span>' . $password . '</span><br>';
			$message .= __( 'Make sure to copy your new key now as it will be hidden once you leave this page.', 'syncspider' );
			wp_send_json_success( $message );
		}

		wp_send_json_error( __( 'Something is wrong.', 'syncspider' ) );
	}

	/**
	 * Render addons page
	 */
	public function settings_addons() {
		$active_settings = SYSP_check::active_addons();
		// 'name', 'description', 'image', 'dir', 'file', 'available',
		$all_addons      = array();
		$all_addons      = apply_filters( 'syncspider_admin_addons', $all_addons );
		$available_count = 0;
		?>

        <div>
            <div id="syncspider-admin-addons" class="syncspider-admin-addons">
				<?php
				if ( ! empty( $all_addons ) ) {
					foreach ( $all_addons as $addon_key => $addon_data ) {
						$style = "";
						if ( ! $addon_data['available'] ) {
							$style = 'style="opacity: 0.6;"';
						}
						?>
                        <div class="syncspider-admin-addons__one" <?= $style ?>>
                            <div class="syncspider-admin-addons__img">
								<?php
								if ( ! empty( $addon_data['image'] ) ) { ?>
                                    <img src="<?= $addon_data['image']; ?>" loading="lazy">
								<?php } ?>
                            </div>
                            <label>
								<?php if ( $addon_data['available'] ) {
									$available_count ++;
									?>
                                    <input name="<?= $addon_data['dir']; ?>" type="checkbox"
                                           value="1" <?php checked( true, isset( $active_settings[ $addon_data['dir'] ] ), true ); ?>
                                           autocomplete="off">
								<?php }
								echo $addon_data['name'];
								?>
                            </label>
                        </div>
						<?php
					}
				}
				?>
            </div>
        </div>
		<?php
		// nonce
		wp_nonce_field( 'syncspider_addons_nonce', 'syncspider_addons_sc' );
		if ( $available_count ) { ?>
            <div class="syncspider-metatabs__btns">
                <div class="syncspider-addons__save syncspider-button__save"
                     data-strsaved="<?php _e( 'Changes saved', 'syncspider' ); ?>"
                     data-strsave="<?php _e( 'Save changes', 'syncspider' ); ?>"
                     data-strsaving="<?php _e( 'Saving changes', 'syncspider' ); ?>"><?php _e( 'Save changes', 'syncspider' ); ?></div>
            </div>
		<?php }
	}

	/**
	 * Ajax save Addon settings
	 */
	public static function save_addons_settings() {
		check_ajax_referer( 'syncspider_addons_nonce', 'sysp_um' );
		$addon_settings = [];

		if ( isset( $_POST['addons'] ) && is_array( $_POST['addons'] ) ) {
			$all_addons = SYSP_check::all_addons();
			$new_addons = $_POST['addons'];

			foreach ( $new_addons as $addon ) {
				if ( in_array( $addon['name'], $all_addons ) && $addon['checked'] === 'true' ) {
					$addon_settings[ $addon['name'] ] = 1;
				}
			}
		}

		update_option( 'syncspider_settings_addons', $addon_settings, true );
		wp_send_json_success();
	}


	/**
	 * Ajax callback for general settings.
	 */
	public function update_general_settings() {
		check_ajax_referer( 'syncspider_general_settings_nonce', 'sysp_um' );
		if ( current_user_can( 'activate_plugins' ) && isset( $_POST['postmeta'] ) && isset( $_POST['usermeta'] ) ) {
			if ( $_POST['postmeta'] == 'post_registered' ) {
				$postmeta = 'registered';
			} elseif ( $_POST['postmeta'] == 'post_all' ) {
				$postmeta = 'all';
			} else {
				$postmeta = 'default';
			}

			if ( $_POST['usermeta'] == 'user_all' ) {
				$usermeta = 'all';
			} else {
				$usermeta = 'default';
			}

			update_option( 'syncspider_general', array(
				'postmeta' => sanitize_text_field( $postmeta ),
				'usermeta' => sanitize_text_field( $usermeta ),
			), false );
			wp_send_json_success();
		}
		wp_send_json_error();
	}
}


