<?php
/**
 * SyncSpider Woo Admin
 *
 *
 * @class    SYSP_Woo_Admin
 * @version  1.0.0
 */

defined( 'ABSPATH' ) || exit;

class SYSP_Woo_Admin {

	protected $version;

	public function __construct( $version ) {
		$this->version = $version;
		add_filter( 'syncspider_menu_items', array( $this, 'admin_menu' ), 10, 1 );
		add_action( 'admin_init', array( $this, 'woo_settings' ) );

		add_action( 'wp_ajax_syncspider_add_order_status', array( $this, 'add_new_order_status' ) );
		add_action( 'wp_ajax_syncspider_remove_order_status', array( $this, 'remove_order_status' ) );

	}

	public function admin_menu( $menu_items ) {
			$menu_items[1000] = array(
				'title'    => __( 'Woocommerce', 'syncspider' ),
				'slug'     => 'woocommerce',
				'callback' => '',
				'submenu' => array(
					'general_settings'    => array(
						'name'     => __( 'General', 'syncspider' ),
						'callback' => array( $this, 'settings_woo' ),
					),
					'order_statuses' => array(
						'name'     => __( 'Order statuses', 'syncspider' ),
						'callback' => array( $this, 'order_statuses' ),
					),
				),
			);

		return $menu_items;
	}

	public function woo_settings() {
		if ( ! wp_doing_ajax() ) {
			add_settings_section(
				'syncspider_woo_settings',
				'',
				'',
				'syncspider_woo_settings'
			);

			register_setting(
				'syncspider_woo_fields',
				'syncspider_settings_woo',
				array( 'sanitize_callback' => array( $this, 'sanitize_woo_settings' ) )
			);

			add_settings_field(
				'syncspider_all_woo_settings',
				'',
				array( $this, 'woo_fields' ),
				'syncspider_woo_settings',
				'syncspider_woo_settings'
			);
		}
	}

	/**
	 * @param $settings
	 *
	 * @return mixed
	 */
	public function sanitize_woo_settings( $settings ) {

		foreach ( $settings as $key => &$set ) {

			if ( '0' === $set ) {
				unset( $settings[ $key ] );
				continue;
			}

			if ( $key == 'new_stock_number' ||  $key == 'product_pdf_priority' ) {
				if ( '' === $set ) {
					continue;
				}
				if ( ! is_numeric( $set ) ) {
					$set = intval( $set );
				}
				$set = abs( $set );
			} elseif ( $key == 'field_images_fields' ) {
				$set = sanitize_text_field( trim( str_replace( " ", "", $set ) ) );
			} elseif ( $key == 'product_pdf_tab_title' ) {
				$set = sanitize_text_field( $set );
			} else {
				$set = 1;
			}
		}

		return $settings;
	}


	public function woo_fields() {
		$woo_settings = get_option( 'syncspider_settings_woo' );
		?>

        <h3><?php _e( 'Product import settings', 'syncspider' ); ?></h3>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_image_checking" name="syncspider_settings_woo[image_checking]"
                       value="1" <?php checked( true, isset( $woo_settings['image_checking'] ), true ); ?> />
				<?php _e('Activate image control', 'syncspider'); ?>
            </label>
            <p style="margin-bottom: 10px;"><em>Try to prevent image duplicates on product level. Allow upload of valid image without extension.</em></p>

            <div id="syncspider_woo_woo_image_checking_fields" style="margin-bottom: 10px;" class="syncspider-subfield <?php echo (isset( $woo_settings['image_checking'] ) ? "" : "syncspider-hide"); ?>">
                <label>
                    <input type="checkbox" id="syncspider_woo_image_prevent_update" name="syncspider_settings_woo[image_prevent_update]"
                           value="1" <?php checked( true, isset( $woo_settings['image_prevent_update'] ), true ); ?> />
		            <?php _e( 'Only upload images on product creation', 'syncspider' ); ?>
                </label>
                <p style="margin-bottom: 10px;"><em>Completely prevent image uploads on batch update rest request.</em></p>
                <div id="syncspider_woo_image_check_id_fields" class="<?php echo( isset( $woo_settings['image_prevent_update'] ) ? "syncspider-hide" : "" ); ?>" style="display: none; margin-bottom: 10px;">
                    <label>
                        <input type="checkbox" id="syncspider_woo_image_check_id" name="syncspider_settings_woo[image_check_id]"
                               value="1" <?php checked( true, isset( $woo_settings['image_check_id'] ), true ); ?> />
			            <?php _e( 'Check image for duplicates if id of existing image is being sent on product updates', 'syncspider' ); ?>
                    </label>
                </div>
                <div>
                    <label>
                        <input type="checkbox" id="syncspider_woocommerce_rest_suppress_image_upload_error" name="syncspider_settings_woo[rest_suppress_image_upload_error]"
                               value="1" <?php checked( true, isset( $woo_settings['rest_suppress_image_upload_error'] ), true ); ?> />
			            <?php _e( 'Suppress image upload errors', 'syncspider' ); ?>
                    </label>
                </div>
                <div>
                    <label>
                        <input type="checkbox" id="syncspider_woo_ignore_content-length" name="syncspider_settings_woo[ignore_content-length]"
                               value="1" <?php checked( true, isset( $woo_settings['ignore_content-length'] ), true ); ?> />
			            <?php _e( 'Only compare image names, ignore content-length', 'syncspider' ); ?>
                    </label>
                </div>
            </div>
        </fieldset>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_product_pdf" name="syncspider_settings_woo[product_pdf]"
                       value="1" <?php checked( true, isset( $woo_settings['product_pdf'] ), true ); ?> />
				<?php _e( 'Activate pdf section on product', 'syncspider' ); ?>
            </label>
        </fieldset>
        <fieldset id="syncspider_woo_checked_product_pdf" class="syncspider-subfield <?php echo( isset( $woo_settings['product_pdf'] ) ? "" : "syncspider-hide" ); ?>">
            <div>
                <label>
                    <?php _e( 'Priority', 'syncspider' ); ?>
                    <input type="number" id="syncspider_product_pdf_priority"
                           name="syncspider_settings_woo[product_pdf_priority]"
                           value="<?php echo esc_attr( $woo_settings['product_pdf_priority'] ?? "" ); ?>" />
                </label>
                <p style="margin-bottom: 10px;"><em><?php _e( 'Position of menu item in product tabs. Default 100.', 'syncspider' ); ?></em></p>
            </div>
            <div>
                <label>
                    <input type="checkbox" id="syncspider_woo_product_pdf_not_original"
                           name="syncspider_settings_woo[product_pdf_not_original]"
                           value="1" <?php checked( true, isset( $woo_settings['product_pdf_not_original'] ), true ); ?> />
					<?php _e( 'Dont use original filename', 'syncspider' ); ?>
                </label>
                <p style="margin-bottom: 10px;"><em>Create name from mapped 'name' and product name. Useful when incoming file names are hash.</em></p>
            </div>
            <div>
                <label>
                    <input type="checkbox" id="syncspider_woo_product_pdf_sequence"
                           name="syncspider_settings_woo[product_pdf_sequence]"
                           value="1" <?php checked( true, isset( $woo_settings['product_pdf_sequence'] ), true ); ?> />
					<?php _e( 'Do not overwrite PDF-s with each request', 'syncspider' ); ?>
                </label>
            </div>
            <div>
                <label>
					<?php _e( 'Product tab title', 'syncspider' ); ?>
                    <input type="text" id="syncspider_woo_product_pdf_tab_label" name="syncspider_settings_woo[product_pdf_tab_title]"
                           value="<?php echo esc_attr( $woo_settings['product_pdf_tab_title'] ?? "" ); ?>"/>
                </label>
            </div>
            <div>
                <label>
					<?php _e( 'Name of Download-Links when not set:', 'syncspider' ); ?>
                    <select id="syncspider_woo_product_pdf_name" name="syncspider_settings_woo[product_pdf_name]" autocomplete="off">
                        <option value="0">Product name</option>
                        <option value="1" <?php selected( isset( $woo_settings['product_pdf_name'] ) ); ?>>Filename</option>
                    </select>
                </label>
            </div>
        </fieldset>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_decimal_stock" name="syncspider_settings_woo[decimal_stock]"
                       value="1" <?php checked( true, isset( $woo_settings['decimal_stock'] ), true ); ?> />
				<?php _e( 'Allow decimal stock', 'syncspider' ); ?>
            </label>
        </fieldset>


        <h4><?php _e( 'Variable product stock control', 'syncspider' ); ?></h4>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_new_stock" name="syncspider_settings_woo[new_stock]"
                       value="1" <?php checked( true, isset( $woo_settings['new_stock'] ), true ); ?> />
				<?php _e( 'Overwrite parent stock on import', 'syncspider' ); ?>
            </label>
        </fieldset>

        <fieldset id="syncspider_woo_new_stock_number_wrapper" class="syncspider-subfield <?php echo( isset( $woo_settings['new_stock'] ) ? "" : "syncspider-hide" ); ?>">
            <label>
				<?php _e( 'Value to set', 'syncspider' ); ?>
                <input type="number" min="0" step="0.001" id="syncspider_woo_new_stock_number" name="syncspider_settings_woo[new_stock_number]"
                       value="<?php echo esc_attr( $woo_settings['new_stock_number'] ?? "" ) ?>"/>
            </label>
            <p><em>If left blank, parent stock will become sum of all variations stock.</em></p>
        </fieldset>


        <h4><?php _e( 'Clean shortcodes & visual editor markup', 'syncspider' ); ?></h4>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_desc_out" name="syncspider_settings_woo[desc_out]"
                       value="1" <?php checked( true, isset( $woo_settings['desc_out'] ), true ); ?> />
				<?php _e( 'From description during export', 'syncspider' ); ?>
            </label>
        </fieldset>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_short_desc_out" name="syncspider_settings_woo[short_desc_out]"
                       value="1" <?php checked( true, isset( $woo_settings['short_desc_out'] ), true ); ?> />
				<?php _e( 'From short description during export', 'syncspider' ); ?>
            </label>
        </fieldset>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_desc_in" name="syncspider_settings_woo[desc_in]"
                       value="1" <?php checked( true, isset( $woo_settings['desc_in'] ), true ); ?> />
				<?php _e( 'From description during import', 'syncspider' ); ?>
            </label>
        </fieldset>
        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_short_desc_in"
                       name="syncspider_settings_woo[short_desc_in]"
                       value="1" <?php checked( true, isset( $woo_settings['short_desc_in'] ), true ); ?> />
				<?php _e( 'From short description during import', 'syncspider' ); ?>
            </label>
        </fieldset>

        <h3><?php _e( 'Additional settings', 'syncspider' ); ?></h3>

        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_customer_number" name="syncspider_settings_woo[customer_number]"
                       value="1" <?php checked( true, isset( $woo_settings['customer_number'] ), true ); ?> />
				<?php _e( "Activate customer number field", 'syncspider' ); ?>
            </label>
            <p style="margin-bottom: 10px;">
                <em>New field 'Customer number' will be available in user edit screen and via rest api.<br>
                    Customer number will be saved to order meta on order creation.
                </em>
            </p>
        </fieldset>

        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_save_regular_line_item" name="syncspider_settings_woo[save_regular_line_item]"
                       value="1" <?php checked( true, isset( $woo_settings['save_regular_line_item'] ), true ); ?> />
				<?php _e( 'Save regular product price on line item meta', 'syncspider' ); ?>
            </label>
        </fieldset>

        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_order_emails" name="syncspider_settings_woo[order_emails]"
                       value="1" <?php checked( true, isset( $woo_settings['order_emails'] ), true ); ?> />
				<?php _e( 'Prevent email sending on order import', 'syncspider' ); ?>
            </label>
        </fieldset>


        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_variation_filter" name="syncspider_settings_woo[variation_filter]"
                       value="1" <?php checked( true, isset( $woo_settings['variation_filter'] ), true ); ?> />
				<?php _e( "Enable variation filters.", 'syncspider' ); ?>
				<?php _e( 'When activated, attribute filters will include only attributes used in <em>enabled</em> variations.', 'syncspider' ); ?>
            </label>
        </fieldset>

        <fieldset id="syncspider_woo_all_pa" class="syncspider-subfield <?php echo( isset( $woo_settings['variation_filter'] ) ? "" : "syncspider-hide" ); ?>">
            <div><?php _e( "Select attributes:", "syncspider" ); ?></div>
			<?php
			$all_pa = wc_get_attribute_taxonomies();
			if ( $all_pa ) {
				foreach ( $all_pa as $pa ) {
					echo '<label class="syncspider-pa-label">';
					echo $pa->attribute_label;
					echo '<input type="checkbox" class="syncspider-pa-checkbox" name="syncspider_settings_woo[pa_' . $pa->attribute_name . ']" value="1" ' . checked( true, isset( $woo_settings[ 'pa_' . $pa->attribute_name ] ), false ) . ' />';
					echo '</label>';
				}
			}
			?>
        </fieldset>

        <fieldset>
            <label>
                <input type="checkbox" id="syncspider_woo_field_images" name="syncspider_settings_woo[field_images]"
                       value="1" <?php checked( true, isset( $woo_settings['field_images'] ), true ); ?> />
				<?php _e( "Enable image upload from custom field for product", 'syncspider' ); ?>
            </label>
        </fieldset>

        <fieldset id="syncspider_woo_field_images_selected_fields" class="syncspider-subfield <?php echo( isset( $woo_settings['field_images'] ) ? "" : "syncspider-hide" ); ?>">
            <label>
				<?php _e( 'Image field', 'syncspider' ); ?>
                <input type="text" id="syncspider_woo_field_images_fields" name="syncspider_settings_woo[field_images_fields]"
                       value="<?php echo esc_attr( $woo_settings['field_images_fields'] ?? "" ); ?>"/>
            </label>
            <p><em></em></p>
        </fieldset>




		<?php
	}


	public function settings_woo() {
		?>
        <form method="post" action="options.php">
			<?php
			settings_fields( 'syncspider_woo_fields' ); // Output nonce, action, and option_page fields for a settings page.
			do_settings_sections( 'syncspider_woo_settings' ); // Print our settings
			submit_button();
			?>
        </form>
		<?php
	}


	public function order_statuses() {
		$order_statuses = get_option( 'syncspider_order_statuses' );

		wp_nonce_field( 'syncspider_order_status_nonce', 'syncspider_order_status_sc' );

		?>

        <h3><?php _e('Additional order statuses', 'syncspider'); ?></h3>
		<?php
		if ( $order_statuses ) { ?>
            <div class="syncspider_status-registered">
                <div></div>
				<?php
				foreach ( $order_statuses as $status_key => $status_name ) {
				    $order_number = wc_get_orders( [ 'status' => $status_key,  'limit' => 1 ] );

				    ?>
                    <div style="margin-bottom: 10px;">
                        <div style="border-bottom: 1px solid #ddd; padding-bottom: 10px; max-width: 260px; display: flex; align-items: center; font-weight: bold;justify-content: space-between;"
                             class="syncspider_status-remove" data-key="<?= $status_key; ?>" data-name="<?= $status_name; ?>">
                            <span style="width: 160px; padding-right: 20px"><?= $status_name; ?></span>
                            <?php
                            if($order_number) { ?>
                                <div>There are orders with this status.</div>
                            <?php } else { ?>
                                <div class="button">Remove</div>
                          <?php  }  ?>


                        </div>
                    </div>
				<?php } ?>
            </div>
		<?php } ?>
        <div style="margin-bottom: 30px;"></div>
        <div class="button syncspider_add-new-status"><?php _e('Add new', 'syncspider'); ?></div>
        <div class="syncspider_new-status" style="display: none;">
            <div style="display: inline-block; border: 1px solid rgb(221, 221, 221); padding: 10px;">

                <input id="syncspider_new_status_field" type="text" value="" placeholder="New order status name">
                <p><?php _e('Please use only letters and spaces.', 'syncspider'); ?></p>

                <div id="syncspider_add-status-message" style="margin-bottom: 15px; color: red; display: none;"></div>

                <div id="syncspider_add-status" class="syncspider-button__save"
                     data-strsaved="<?php _e( 'Saved', 'syncspider' ) ?>"
                     data-strsave="<?php _e( 'Submit', 'syncspider' ) ?>"
                     data-strsaving="<?php _e( 'Adding new', 'syncspider' ) ?>"><?php _e( 'Submit', 'syncspider'
		            ) ?>
                </div>
                <div id="syncspider_add-status-cancel" class="button"><?php _e('Cancel', 'syncspider'); ?></div>
            </div>
        </div>


        <script>
            jQuery(document).ready(function ($) {
                var sc = $('#syncspider_order_status_sc').val();

                $('.syncspider_add-new-status').on('click', function (){
                    $(this).hide();
                    $('.syncspider_new-status').show();
                });

                $('#syncspider_add-status-cancel').on( 'click', function (){
                    $('.syncspider_new-status').hide();
                    $('.syncspider_add-new-status').show();
                });

                $('#syncspider_add-status').on('click', function (){
                    var button = $(this);
                    var msg = $('#syncspider_add-status-message');
                    $.ajax({
                        type: 'POST',
                        dataType: 'json',
                        url: ajaxurl,
                        data: {
                            'action': 'syncspider_add_order_status',
                            'status': $('#syncspider_new_status_field').val(),
                            'sysp_um': sc
                        },
                        beforeSend: function () {
                            button.addClass('syncspider_btn_saving');
                            msg.html('').hide();
                        },
                        success: function (response) {
                            if (response.success) {
                                msg.css('color', '#000');
                                msg.html(response.data).show();
                                window.location.reload();
                            } else {
                                msg.css('color', 'red');
                                msg.html(response.data).show();
                            }
                        },
                        error: function (response) {
                            msg.html('Error').show();
                        },
                        complete: function () {
                            button.removeClass('syncspider_btn_saving');
                        }
                    });
                });

                $('.syncspider_status-remove .button').on('click', function () {
                    var slug = $(this).closest('.syncspider_status-remove').data('key');
                    var name = $(this).closest('.syncspider_status-remove').data('name');
                    let conf = confirm("<?php _e('Are you the sure you want to remove order status', 'syncspider') ?>\n'" + name + "'?");
                    if (conf) {
                        $.ajax({
                            type: 'POST',
                            dataType: 'json',
                            url: ajaxurl,
                            data: {
                                'action': 'syncspider_remove_order_status',
                                'status': slug,
                                'sysp_um': sc
                            },
                            success: function (response) {
                                if (response.success) {
                                    window.location.reload();
                                } else {
                                    alert(response.data);
                                }
                            },
                            error: function (response) {
                                alert('Error');
                            }
                        });
                    }
                });
            });
        </script>

		<?php
	}


    /**
	 * Ajax callback for status generation
     */
	public function add_new_order_status() {
		check_ajax_referer( 'syncspider_order_status_nonce', 'sysp_um' );
		if ( current_user_can( 'activate_plugins' ) && isset( $_POST['status'] ) && preg_match( '/^([a-zA-Z]+ )*[a-zA-Z]+$/', $_POST['status'] ) && strlen( $_POST['status'] ) <= 17 ) {
			$new_status          = $_POST['status'];
			$registered_statuses = wc_get_order_statuses();


			if ( in_array( strtolower( $new_status ), array_map( 'strtolower', $registered_statuses ) ) ) {
				wp_send_json_error( __( 'Order status already exists.', 'syncspider' ) );
			}

			$new_status_slug = sanitize_title_with_dashes( $new_status );

			$order_statuses = get_option( 'syncspider_order_statuses' );
			if ( ! $order_statuses ) {
				$order_statuses = [];
			}

			if ( count( $order_statuses ) > 10 ) {
				wp_send_json_error( __( 'Limit reached. Please remove some of the items before adding new one.', 'syncspider' ) );
			}

			$order_statuses[ $new_status_slug ] = $new_status;

			update_option( 'syncspider_order_statuses', $order_statuses, true );
			wp_send_json_success( __( 'Order status added.', 'syncspider' ) );
		}
		wp_send_json_error( __( 'Please enter valid order status name.', 'syncspider' ) );
	}

    /**
	 * Ajax callback for status removal
     */
	public function remove_order_status() {
        check_ajax_referer( 'syncspider_order_status_nonce', 'sysp_um' );

        if ( current_user_can( 'activate_plugins' ) && isset( $_POST['status'] )) {
           $order_statuses = get_option( 'syncspider_order_statuses' );
            unset($order_statuses[$_POST['status']]);
            update_option( 'syncspider_order_statuses', $order_statuses, true );
            wp_send_json_success();
        }
        wp_send_json_error();
	}

}