<?php

defined( 'ABSPATH' ) || exit;

class SYSP_auth {

	public function __construct() {
		$rest_api = SYSP_check::check_is_rest_api();

		if ( $rest_api ) {
			// enable woocommerce auth for syncspider endpoints
			if ( $rest_api === 'syncspider' ) {
				add_filter( 'woocommerce_rest_is_request_to_rest_api', '__return_true' );
			}

			add_filter( 'rest_authentication_errors', array( $this, 'json_token_auth_error' ) );
			add_filter( 'determine_current_user', array( $this, 'json_token_auth_handler' ), 100 );
		}
	}

	public function json_token_auth_handler( $user_id ) {

		global $wp_json_token_auth_error;
		$wp_json_token_auth_error = null;

		if( ! SYSP_check::is_auth() ) {

			if ( ! empty( $user_id ) ) {

				$user = get_user_by( 'id', $user_id );
				remove_filter( 'determine_current_user', array( $this, 'json_token_auth_handler' ), 100 );
				if ( $user->has_cap('activate_plugins' ) ) {
					SYSP_check::set_auth();
				}
				add_filter( 'determine_current_user', array( $this, 'json_token_auth_handler' ), 100 );

			} elseif ( SYSP_check::check_token() ) {

				$user = get_user_by( 'id', get_option( 'wp_syncspider_user_id' ) );
				/**
				 * In multi-site, wp_authenticate_spam_check filter is run on authentication. This filter calls
				 * get_currentuserinfo which in turn calls the determine_current_user filter. This leads to infinite
				 * recursion and a stack overflow unless the current function is removed from the determine_current_user
				 * filter during authentication.
				 */
				remove_filter( 'determine_current_user', array( $this, 'json_token_auth_handler' ), 100 );
				wp_clear_auth_cookie();
				wp_set_current_user( $user->ID );

				add_filter( 'determine_current_user', array( $this, 'json_token_auth_handler' ), 100 );

				if ( is_wp_error( $user ) ) {
					$wp_json_token_auth_error = $user;

					return null;
				}
				$wp_json_token_auth_error = true;

				SYSP_check::set_auth();

				return $user->ID;
			}
		}

		return $user_id;
	}

	public function json_token_auth_error( $error ) {
		// Passthrough other errors
		if ( ! empty( $error ) ) {
			return $error;
		}
		global $wp_json_token_auth_error;
		return $wp_json_token_auth_error;
	}

}