<?php

defined( 'ABSPATH' ) || exit;

class SYSP_check {

	private static $token_auth, $is_woo, $is_dokan, $is_gm, $is_auth, $is_rest_api, $is_ssl, $all_active_plugins, $active_addons, $all_addons;

	/**
	 * Check if incoming data has valid SyncSpider token
	 *
	 * @return bool
	 */
	public static function check_token() {

		// only check once
		if ( isset( self::$token_auth ) ) {
			return self::$token_auth;
		}

		if ( isset( $_SERVER['HTTP_TOKEN'] ) ) {
			$token = $_SERVER['HTTP_TOKEN'];

			$sysp_hash = get_option( 'wp_syncspider_api_hash' );

			if ( ! empty( $sysp_hash ) && wp_check_password( $token, $sysp_hash ) ) {
				self::$token_auth = true;
			} else {
				self::$token_auth = false;
			}
		} else {
			self::$token_auth = false;
		}

		return self::$token_auth;
	}


	/**
	 * Check if either token was valid or woocommerce rest auth passed.
	 *
	 * @return bool
	 */
	public static function check_rest_view() {
		if ( self::is_auth() ) {
			return true;
		}

		return false;
	}


	/**
	 * Check if is request to REST API. wp-json/wp/v2/syncspider && wp-json/wp/v2/syncspider
	 *
	 * @return bool
	 */
	public static function check_is_rest_api() {

		// only check once
		if ( isset( self::$is_rest_api ) ) {
			return self::$is_rest_api;
		}

		if ( empty( $_SERVER['REQUEST_URI'] ) ) {
			self::$is_rest_api = false;

			return false;
		}

		// if site is installed in folder
		$subfolder = site_url( '', 'relative' );

		$wp_api_routes     = $subfolder . '/wp-json/wp/v2/';
		$sysp_api_route_v2 = $subfolder . '/wp-json/wp/v2/syncspider';
		// todo: suggest removal of v3 from app.syncspider
		$sysp_api_routes_v3 = $subfolder . '/wp-json/wp/v3/syncspider';


		if ( 0 === strpos( $_SERVER['REQUEST_URI'], $sysp_api_route_v2 ) || 0 === strpos( $_SERVER['REQUEST_URI'], $sysp_api_routes_v3 ) ) {
			$is_rest_api_request = 'syncspider';
		} elseif ( 0 === strpos( $_SERVER['REQUEST_URI'], $wp_api_routes ) ) {
			$is_rest_api_request = 'wp';
		} else {
			$is_rest_api_request = false;
		}

		self::$is_rest_api = $is_rest_api_request;

		return self::$is_rest_api;
	}


	/**
	 * Check ssl
	 *
	 * @return mixed
	 */
	public static function check_ssl() {
		// only check once
		if ( isset( self::$is_ssl ) ) {
			return self::$is_ssl;
		}

		self::$is_ssl = is_ssl();

		return self::$is_ssl;
	}

	/**
	 * Set if auth is true
	 */
	public static function set_auth() {
		if ( ! isset( self::$is_auth ) ) {
			self::$is_auth = true;
		}
	}

	/**
	 * @return bool
	 */
	public static function is_auth() {
		if ( isset( self::$is_auth ) ) {
			return true;
		}

		return false;
	}


	/**
	 * Retreive all active plugins, cache
	 *
	 * @return mixed|void
	 */
	public static function get_active_plugins() {
		if ( isset( self::$all_active_plugins ) ) {
			return self::$all_active_plugins;
		}
		$active_multi = [];
		if ( is_multisite() ) {
			$active_multi = get_site_option( 'active_sitewide_plugins' );
		}

		$active = array_flip(get_option( 'active_plugins' ));

		self::$all_active_plugins = $active + $active_multi;

		return self::$all_active_plugins;
	}


	/**
	 * Check if woocommerce is installed and active
	 *
	 * @return bool
	 */
	public static function check_woo() {

		// only check once
		if ( isset( self::$is_woo ) ) {
			return self::$is_woo;
		}

		self::$is_woo = self::check_plugin( 'woocommerce/woocommerce.php' );

		return self::$is_woo;
	}

	/**
	 * Check if Dokan is installed and active
	 * Only check dokan-lite, pro depends on it.
	 *
	 * @return bool
	 */
	public static function check_dokan() {

		if ( isset( self::$is_dokan ) ) {
			return self::$is_dokan;
		}

		self::$is_dokan = self::check_plugin( 'dokan-lite/dokan.php' );

		return self::$is_dokan;
	}

	/**
	 * Check if German Market is installed and active
	 *
	 * @return bool
	 */
	public static function check_gm() {
		if ( isset( self::$is_gm ) ) {
			return self::$is_gm;
		}

		self::$is_gm = self::check_plugin( 'woocommerce-german-market/WooCommerce-German-Market.php' );

		return self::$is_gm;
	}

	/**
	 * Check is plugin is active
	 *
	 * @param $plugin_path
	 *
	 * @return bool
	 */
	public static function check_plugin( $plugin_path ) {

		if ( isset( self::get_active_plugins()[$plugin_path] ) ) {
			return true;
		}

		return false;
	}


	/**
	 * Get activated Syncspider addons
	 * @return false|mixed|void
	 */
	public static function active_addons() {
		if ( isset( self::$active_addons ) ) {
			return self::$active_addons;
		}

		$addons              = get_option( 'syncspider_settings_addons' );
		self::$active_addons = $addons;

		return $addons;
	}

	/**
	 * Get all addons
	 * @return array|false
	 */
	public static function all_addons() {
		if ( isset( self::$all_addons ) ) {
			return self::$all_addons;
		}

		$all_addons_files = scandir( SYNCSPIDER_ADDONS_DIR );

		$addon_dirs = [];
		if ( ! empty( $all_addons_files ) ) {
			// remove files and root folders from filelist
			$addon_dirs = array_filter( $all_addons_files, function ( $e ) {
				return ( strpos( $e, "." ) === false );
			} );
		}
		self::$all_addons = $addon_dirs;

		return $addon_dirs;
	}
}