<?php

class SYSP_Woo_Addons {

	private $woo_settings;
	private $order_statuses;

	/**
	 * SYSP_Woo_Addons constructor.
	 */
	public function __construct() {

		$this->woo_settings = get_option( 'syncspider_settings_woo' );

		if ( $this->opt_is( 'desc_out' ) || $this->opt_is( 'short_desc_out' ) ) {
			add_filter( 'rest_post_dispatch', array( $this, 'get_short_description' ), 10, 3 );
		}

		if ( $this->opt_is( 'desc_in' ) || $this->opt_is( 'short_desc_in' ) || $this->opt_is( 'product_pdf' ) || $this->opt_is( 'image_checking' ) || $this->opt_is( 'field_images' ) ) {
			add_filter( 'rest_dispatch_request', array( $this, 'addons_rest_dispatch' ), 10, 4 );
		}

		if ( $this->opt_is( 'image_checking' ) ) {
			add_filter( 'wp_handle_sideload_prefilter', array( $this, 'add_ext_if_no_extension' ) );

			if ( $this->opt_is( 'rest_suppress_image_upload_error' ) ) {
				add_filter( 'woocommerce_rest_suppress_image_upload_error', '__return_true', 11 );
			}
		}

		// additional action for image in meta field
		if ( $this->opt_is( 'field_images' ) ) {
			add_action( 'woocommerce_rest_insert_product_object', array( $this, 'update_field_images' ), 10, 3 );
		}

		// additional action and filter for pdf product.
		if ( $this->opt_is( 'product_pdf' ) ) {
			add_action( 'woocommerce_rest_insert_product_object', array( $this, 'update_product_pdf' ), 10, 3 );
			// Adding Download section on single product
			add_filter( 'woocommerce_product_tabs', array( $this, 'syncspider_product_pdf_show_tabs' ) );
		}

		if ( $this->opt_is( 'order_emails' ) ) {
			if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
				add_action( 'woocommerce_email', array( $this, 'unhook_emails' ) );
			}
		}

		if ( $this->opt_is( 'decimal_stock' ) ) {
			add_action( 'plugins_loaded', function () {
				remove_filter( 'woocommerce_stock_amount', 'intval' );
				add_filter( 'woocommerce_stock_amount', 'floatval' );
			} );
		}

		if ( $this->opt_is( 'new_stock' ) ) {
			add_action( 'woocommerce_update_product', array( $this, 'variable_stock' ), 10, 2 );
		}

		if ( $this->opt_is( 'save_regular_line_item' ) ) {
			add_action( 'woocommerce_checkout_create_order_line_item', array( $this, 'save_regular_line_item' ), 10, 4 );
			add_filter( 'syncspider_add_product_meta', function ( $meta ) {
				$meta[] = array(
					'slug'  => '_syncspider_original_price',
					'label' => 'SyncSpider original product price',
				);

				return $meta;
			} );
		}

		if ( $this->opt_is( 'variation_filter' ) ) {
			add_filter( 'get_terms', array( $this, 'variation_filter' ), 10, 4 );
		}

		if ( $this->opt_is( 'customer_number' ) ) {

			add_action( 'show_user_profile', array( $this, 'profile_fields' ), 10 );
			add_action( 'edit_user_profile', array( $this, 'profile_fields' ), 10 );


			add_action( 'personal_options_update', array( $this, 'save_profile_fields' ) );
			add_action( 'edit_user_profile_update', array( $this, 'save_profile_fields' ) );


			add_filter( 'manage_users_columns', array( $this, 'user_table' ) );
			add_filter( 'manage_users_custom_column', array( $this, 'user_table_row' ), 10, 3 );

			add_filter( 'syncspider_add_user_meta', array( $this, 'user_meta' ) );
			add_filter( 'syncspider_add_order_meta', array( $this, 'user_meta' ) );

			add_action( 'woocommerce_checkout_create_order', array( $this, 'create_order' ), 20, 2 );
			add_action( 'woocommerce_admin_order_data_after_order_details', array( $this, 'display_order_meta' ) );
		}

		add_action( 'init', array( $this, 'register_new_post_status' ) );
		add_filter( 'wc_order_statuses', array( $this, 'new_order_statuses' ), 20 );

	}

	/**
	 * Intercept product / variation rest request and save images / pdfs  to meta data. Remove from request.
	 *
	 * @param mixed $response Dispatch result
	 * @param WP_REST_Request $request Request used to generate the response.
	 * @param string $route Route matched for the request.
	 * @param array $handler Route handler used for the request.
	 *
	 * @return mixed
	 */
	public function addons_rest_dispatch( $response, $request, $route, $handler ) {


		// checking if route is batch for images and pdf
		if ( ( 'POST' == $request->get_method() ) && ( '/wc/v3/products/batch' === $route || '/wc/v3/products/(?P<product_id>[\d]+)/variations/batch' === $route ) ) {

			// checking if route is batch for pdf

			// get request body
			$body = json_decode( $request->get_body(), true );

			// if body is not json or is invalid, continue
			if ( $body === null ) {
				return $response;
			}

			// filter body in order to allow external body changes
			$body = apply_filters( 'syncspider_incoming_batch_body', $body, $route, $request );

			if ( ! empty( $body ) ) {
				foreach ( $body as $method => &$methodBody ) {
					// only for update and create methods
					if ( 'delete' !== $method ) {
						foreach ( $methodBody as &$b ) {
							// products route - pdf, desc, images
							if ( '/wc/v3/products/batch' === $route ) {

								// description
								if ( $this->opt_is( 'desc_in' ) && ! empty( $b['description'] ) ) {
									$b['description'] = $this->clean_text( $b['description'] );
								}

								// short description
								if ( $this->opt_is( 'short_desc_in' ) && ! empty( $b['short_description'] ) ) {
									$b['short_description'] = $this->clean_text( $b['short_description'] );
								}

								// product pdf
								if ( $this->opt_is( 'product_pdf' ) && ! empty( $b['syncspider_product_pdf'] ) ) {
									$pdfs = array(
										'key'   => '_sysp_product_pdf_received',
										'value' => $b['syncspider_product_pdf']
									);
									unset( $b['syncspider_product_pdf'] );

									// maybe 'meta_data' is not set
									$b['meta_data']   = $b['meta_data'] ?? [];
									$b['meta_data'][] = $pdfs;
								}

								// 'images' will be set only on product.
								if ( $this->opt_is( 'image_checking' ) && ! empty( $b['images'] ) ) {
									if ( 'update' === $method && $this->opt_is( 'image_prevent_update' ) ) {
										$bulk_images_value = '';
									} else {
										$bulk_images_value = $b['images'];
									}

									$imgs = array( 'key' => '_sysp_bulk_images', 'value' => $bulk_images_value );
									unset( $b['images'] );

									// maybe 'meta_data' is not set
									$b['meta_data']   = $b['meta_data'] ?? [];
									$b['meta_data'][] = $imgs;
								}

								// Custom image field
								if ( $this->opt_is( 'field_images' ) ) {

									$custom_field_name = $this->opt_val( 'field_images_fields' );

									if ( $custom_field_name && ! empty( $b['meta_data'] ) ) {
										foreach ( $b['meta_data'] as $k => $meta ) {
											if ( isset( $meta['key'] ) && $meta['key'] == $custom_field_name ) {
												$sysp_fields = [];
												// todo: multiple fields
												$sysp_fields[ $custom_field_name ] = $meta['value'];

												$b['meta_data'][ $k ] = array(
													'key'   => '_sysp_product_field_images_received',
													'value' => $sysp_fields,
												);
											}
										}
									}
								}
							} else {
								// variation route - image only
								// 'image' will be set only on variation
								if ( $this->opt_is( 'image_checking' ) && ! empty( $b['image'] ) ) {
									if ( 'update' === $method && $this->opt_is( 'image_prevent_update' ) ) {
										$val = '';
									} else {
										$val = $b['image'];
									}
									unset( $b['image'] );
									$img              = array( 'key' => '_sysp_var_image', 'value' => $val );
									$b['meta_data']   = $b['meta_data'] ?? [];
									$b['meta_data'][] = $img;
								}

							}
						}
					}
				}

				$body = json_encode( $body );
				$request->set_body( $body );
				// callback from $handler
				$response = call_user_func( $handler['callback'], $request );
			}
		}

		return $response;
	}


	public function add_ext_if_no_extension( $file ) {
		if ( pathinfo( $file['name'], PATHINFO_EXTENSION ) ) {
			return $file;
		}
		$real_mime   = wp_get_image_mime( $file['tmp_name'] );
		$mime_to_ext = apply_filters(
			'getimagesize_mimes_to_exts',
			array(
				'image/jpeg' => 'jpg',
				'image/png'  => 'png',
				'image/gif'  => 'gif',
				'image/bmp'  => 'bmp',
				'image/tiff' => 'tif',
				'image/webp' => 'webp',
			)
		);
		if ( ! empty( $mime_to_ext[ $real_mime ] ) ) {
			$file['name'] .= '.' . $mime_to_ext[ $real_mime ];
		}

		return $file;
	}

	public function update_field_images( $object, $request, $creating ) {
		$new_fields      = $object->get_meta( '_sysp_product_field_images_received', true );
		$existing_fields = $object->get_meta( '_sysp_product_field_images_existing', true );

		$key = $this->opt_val( 'field_images_fields' );

		if ( $new_fields ) {
			if ( isset( $existing_fields[ $key ] ) && isset( $new_fields[ $key ] ) && $existing_fields[ $key ] == $new_fields[ $key ] ) {
				delete_post_meta( $object->get_ID(), '_sysp_product_field_images_received' );
			} else {
				update_post_meta( $object->get_ID(), '_sysp_product_field_images_existing', $new_fields );

				if ( ! function_exists( 'download_url' ) ) {
					require_once( ABSPATH . 'wp-admin/includes/file.php' );
				}
				if ( ! function_exists( 'media_upload_tabs' ) ) {
					require_once( ABSPATH . 'wp-admin/includes/media.php' );
				}
				if ( ! function_exists( 'wp_crop_image' ) ) {
					require_once( ABSPATH . 'wp-admin/includes/image.php' );
				}

				$upload = wc_rest_upload_image_from_url( esc_url_raw( $new_fields[ $key ] ) );

				if ( is_wp_error( $upload ) ) {
					if ( ! apply_filters( 'woocommerce_rest_suppress_image_upload_error', false, $upload, 0, array( $new_fields[ $key ] ) ) ) {
						throw new WC_REST_Exception( 'woocommerce_product_image_upload_error', $upload->get_error_message(), 400 );
					} else {
						// todo: internal handle
					}
				} else {
					$attachment_id = wc_rest_set_uploaded_image_as_attachment( $upload, $object->get_id() );
					update_post_meta( $object->get_ID(), $key, wp_get_attachment_url( $attachment_id ) );
					delete_post_meta( $object->get_ID(), '_sysp_product_field_images_received' );
				}
			}
		}
	}

	/**
	 * @param $object
	 * @param $request
	 * @param $creating
	 *
	 * @return mixed
	 */
	public function update_product_pdf( $object, $request, $creating ) {
		$new_pdfs = $object->get_meta( '_sysp_product_pdf_received', true );
		$pdfs     = $object->get_meta( '_sysp_product_pdf', true );

		if ( ! empty( $new_pdfs ) && is_array( $new_pdfs ) ) {
			$pdf_ids = [];

			// including media handling files
			if ( ! function_exists( 'download_url' ) ) {
				require_once( ABSPATH . 'wp-admin/includes/file.php' );
			}
			if ( ! function_exists( 'media_upload_tabs' ) ) {
				require_once( ABSPATH . 'wp-admin/includes/media.php' );
			}
			if ( ! function_exists( 'wp_crop_image' ) ) {
				require_once( ABSPATH . 'wp-admin/includes/image.php' );
			}


			foreach ( $new_pdfs as $pdf ) {

				$url = $pdf['src'];

				//checking existing pdfs
				if ( $pdfs && is_array( $pdfs ) ) {

					// new one by one, add new only if file is new, change name if new provided
					if ( $this->opt_is( 'product_pdf_sequence' ) ) {
						$unique_pdfs          = $pdfs;
						$unique_existing_keys = [];
						foreach ( $pdfs as $key => $p ) {

							if ( strpos( $url, 'https://app.syncspider.com/uploads/custom_api' ) === 0 ) {
								//strip hash
								$pdfurl = $this->maybe_strip_hash( wp_basename( $url ) );
							} else {
								$pdfurl = wp_basename( $url );
							}


							if ( $this->ends_with( $p['original_src'], $pdfurl ) ) {
								//check filesize
								$remote_file_size = 0;
								// file name already uploaded, checking filesize
								$headers = wp_get_http_headers( $url );

								if ( $headers ) {
									$headers = $headers->getAll();

									if ( isset( $headers['content-length'] ) ) {
										$remote_file_size = $headers['content-length'];
									}
								}

								//  same file size, check file name

								if ( $remote_file_size == $p['filesize'] && get_attached_file( $p['id'], true ) ) {

									// it has same source, check name
									if ( isset( $pdf['name'] ) && $pdf['name'] === $p['original_name'] ) {
										// same file is found, do nothing.
//										continue 2;
									} else {
										// change file display name only.
										$unique_pdfs[ $key ]['original_name'] = $pdf['name'] ?? "";
									}
									continue 2;

								}
							}
						}

					} else {
						$existing_key = false;
						if ( strpos( $url, 'https://app.syncspider.com/uploads/custom_api' ) === 0 ) {
							//strip hash
							$pdfurl = $this->maybe_strip_hash( wp_basename( $url ) );
						} else {
							$pdfurl = wp_basename( $url );
						}


						foreach ( $pdfs as $key => $p ) {
							if ( $this->ends_with( $p['original_src'], $pdfurl ) ) {
								$existing_key = $key;
							}
						}

						if ( $existing_key !== false ) {
							$remote_file_size = 0;
							// file name already uploaded, checking filesize
							$headers = wp_get_http_headers( $url );

							if ( $headers ) {
								$headers = $headers->getAll();

								if ( isset( $headers['content-length'] ) ) {
									$remote_file_size = $headers['content-length'];
								}
							}

							// sizes match, set old image todo: option to avoid checking size
							if ( $remote_file_size == $pdfs[ $existing_key ]['filesize'] && get_attached_file( $pdfs[ $existing_key ]['id'], true ) ) {
								$pdf_ids[] = array(
									'id'            => $pdfs[ $existing_key ]['id'],
									'original_src'  => $url,
									'original_name' => $pdf['name'] ?? '',
									'filesize'      => $pdfs[ $existing_key ]['filesize'],
								);

								continue;
							}
						}
					}
				}


				$tmp = download_url( $url );

				if ( is_wp_error( $tmp ) ) {
					@unlink( $tmp );
					// we continue on error, disscuss
					continue;
				}

				/*
				 * Pdf is default
				 */
				$accepted_mimes = array(
					 'pdf' => 'application/pdf',
				);


				$accepted_mimes = apply_filters( 'syncspider_product_pdf_accepted_mimes', $accepted_mimes );

				$file_paths             = pathinfo( $url );
                $file_extension = $file_paths['extension'] ?? '';

				$file_type = mime_content_type( $tmp );
				$file_size = filesize( $tmp );

				if ( isset( $accepted_mimes[ $file_extension ] ) ) {

					$extension = $file_extension;

					if ( $this->opt_is( 'product_pdf_not_original' ) ) {
						$name = sanitize_file_name( $pdf['name'] . "-" . $object->get_name() ) . '.' . $extension;
					} else {
						$name = wp_basename( $url );
					}

					$file_array = array(
						'name'     => $name,
						'tmp_name' => $tmp
					);

					$post_id = $object->get_id();
					add_filter( 'upload_mimes', array( $this, 'pdf_upload_mimes' ), 10 );
					$uploaded_attachment_id = media_handle_sideload( $file_array, $post_id );
					remove_filter( 'upload_mimes', array( $this, 'pdf_upload_mimes' ), 10 );

					if ( $uploaded_attachment_id ) {
						$pdf_ids[] = array(
							'id'            => $uploaded_attachment_id,
							'original_src'  => $url,
							'original_name' => $pdf['name'],
							'filesize'      => $file_size
						);
					}
				}
			}

			if ( $this->opt_is( 'product_pdf_sequence' ) && isset( $unique_pdfs ) ) {
				update_post_meta( $object->get_ID(), '_sysp_product_pdf', array_merge( $unique_pdfs, $pdf_ids ) );
			} else {
				update_post_meta( $object->get_ID(), '_sysp_product_pdf', $pdf_ids );
			}

			//cleanup
			delete_post_meta( $object->get_ID(), '_sysp_product_pdf_received' );
			$object->save();
			wc_delete_product_transients( $object->get_id() );
		}

		return $object;
	}


	/**
	 * Display Download tab if files exist
	 *
	 * @param $tabs
	 *
	 * @return mixed
	 */
	function syncspider_product_pdf_show_tabs( $tabs ) {

		global $product;

		if ( $product ) {
			$pdfs = get_post_meta( $product->get_ID(), '_sysp_product_pdf', true );

			$title = esc_html( $this->opt_val( 'product_pdf_tab_title' ) );

			if ( empty( $title ) ) {
				$title = __( 'Download', 'woocommerce' );
			}

			if ( $pdfs ) {
			    $priority = ( $this->opt_val('product_pdf_priority' ) ? : 100 );
				$tabs['syncspider_download_tabs'] = array(
					'title'    => $title,
					'priority' => (int) $priority,
					'callback' => array( $this, 'syncspider_product_pdf_tabs' )
				);
			}
		}

		return $tabs;
	}

	/**
	 * Print pdf tab
	 */
	public function syncspider_product_pdf_tabs() {
		global $product;

		if ( $product ) {
			$pdfs = get_post_meta( $product->get_ID(), '_sysp_product_pdf', true );

			echo "<ul class='sysp-product__pdf'>";

			if ( $pdfs ) {
				foreach ( $pdfs as $pdf ) {
					$url = wp_get_attachment_url( $pdf['id'] );
					if ( $url ) {
						$name = $pdf['original_name'];
						$class = 'sysp-product__pdf-link ';
						if ( ! $name ) {
							if ( $this->opt_is( 'product_pdf_name' ) ) {

								$file_url = $pdf['original_src'];
								//strip hash - need change.
								if ( strpos( $file_url, 'https://app.syncspider.com/uploads/custom_api' ) === 0 ) {
									$name = $this->maybe_strip_hash( wp_basename( $file_url ) );
								} else {
									$name = wp_basename( $file_url );
								}
							} else {
//								$name = get_the_title( $pdf['id'] );
								$name = $product->get_title();
							}
						} else {
							$class .= sanitize_title( $name );
                        }
						$name = apply_filters('syncspider_product_pdf_name', $name, $pdf, $url );
						echo '<li><a class="' . esc_attr($class) . '" href="' . $url . '" target="_blank" download><span></span>' . esc_html($name) . '</a></li>';
					}
				}
			}
			echo "</ul>";
		}
	}

	/**
	 * Add additional mimes to be able to upload as file.
	 *
	 * @param $mimes
	 *
	 * @return mixed
	 */
	public function pdf_upload_mimes( $mimes ) {
		$additional_mimes = apply_filters( 'syncspider_product_pdf_accepted_mimes', $mimes );

		return $additional_mimes;
	}

	/**
	 * Check if there is hash in filename and strip it.
	 *
	 * @param $filename
	 *
	 * @return false|mixed|string
	 */
	public function maybe_strip_hash( $filename ) {
		$hash = mb_substr( $filename, 0, 33, 'utf-8' );

		if ( strlen( $hash ) === 33 && mb_strlen( $filename ) > 37 && preg_match( '/^[a-f0-9]{32}_$/', $hash ) ) {
			return mb_substr( $filename, 33, null, 'utf-8' );
		}

		return $filename;
	}


	public function unhook_emails( $email_class ) {

		remove_action( 'woocommerce_low_stock_notification', array( $email_class, 'low_stock' ) );
		remove_action( 'woocommerce_no_stock_notification', array( $email_class, 'no_stock' ) );
		remove_action( 'woocommerce_product_on_backorder_notification', array( $email_class, 'backorder' ) );

		// New order emails
		remove_action( 'woocommerce_order_status_pending_to_processing_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_pending_to_completed_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_pending_to_on-hold_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_failed_to_processing_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_failed_to_completed_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_failed_to_on-hold_notification', array(
			$email_class->emails['WC_Email_New_Order'],
			'trigger'
		) );

		// Processing order emails
		remove_action( 'woocommerce_order_status_pending_to_processing_notification', array(
			$email_class->emails['WC_Email_Customer_Processing_Order'],
			'trigger'
		) );
		remove_action( 'woocommerce_order_status_pending_to_on-hold_notification', array(
			$email_class->emails['WC_Email_Customer_Processing_Order'],
			'trigger'
		) );

		// Completed order emails
		remove_action( 'woocommerce_order_status_completed_notification', array(
			$email_class->emails['WC_Email_Customer_Completed_Order'],
			'trigger'
		) );

		// Note emails
		remove_action( 'woocommerce_new_customer_note_notification', array(
			$email_class->emails['WC_Email_Customer_Note'],
			'trigger'
		) );
	}


	/**
	 * @param WP_HTTP_Response $result Result to send to the client. Usually a `WP_REST_Response`.
	 * @param WP_REST_Server $server Server instance.
	 * @param WP_REST_Request $request Request used to generate the response.
	 */

	public function get_short_description( $result, $server, $request ) {

		if ( ! $result->is_error() && 'GET' == $request->get_method() ) {

			$route = $result->get_matched_route();

			if ( $route === '/wc/v3/products/(?P<id>[\d]+)' ) {
				$data = $result->get_data();
				if ( ! empty( $data ) ) {

					if ( $this->opt_is( 'desc_out' ) ) {
						$data['description'] = $this->clean_text( $data['description'] );
					}
					if ( $this->opt_is( 'short_desc_out' ) ) {
						$data['short_description'] = $this->clean_text( $data['short_description'] );
					}

					$result->set_data( $data );

					return $result;
				}
			}

			if ( $route === '/wc/v3/products' ) {
				$data = $result->get_data();
				if ( ! empty( $data ) ) {
					foreach ( $data as &$d ) {
						if ( $this->opt_is( 'desc_out' ) ) {
							$d['description'] = $this->clean_text( $d['description'] );
						}
						if ( $this->opt_is( 'short_desc_out' ) ) {
							$d['short_description'] = $this->clean_text( $d['short_description'] );
						}
					}
					$result->set_data( $data );

					return $result;
				}
			}
		}

		return $result;
	}


	function variable_stock( $id, $product ) {

		if ( $product->is_type( 'variable' ) ) {

			// must be numeric.

			$variations_route = "/wc/v3/products/(?P<product_id>[\d]+)/variations/batch";

			$path = isset( $GLOBALS['wp']->query_vars['rest_route'] ) ? untrailingslashit( $GLOBALS['wp']->query_vars['rest_route'] ) : "";
			if ( empty( $path ) ) {
				$path = '/';
			}

			$match = preg_match( '@^' . $variations_route . '$@i', $path );

			// route is variation batch
			if ( $match ) {
				// remove action to prevent loop
				remove_action( 'woocommerce_update_product', array( $this, 'variable_stock' ), 10 );

				// allow dynamic override
				$incoming_body = json_decode( rest_get_server()->get_raw_data(), true );
				$disable       = apply_filters( 'syncspider_disable_variable_stock', false, $product, $incoming_body );

				if ( ! $disable ) {

					$new_stock_number = $this->opt_val( 'new_stock_number' );

					// if new_stock_number is set dont calculate. stock new_stock_number can be 0
					if ( $new_stock_number !== "" ) {
						// set manage stock anyway
						$product->set_manage_stock( true );

						$product->set_stock_quantity( $new_stock_number );
						$product->save();
					} else {
						// set only if some of variations have 'manage stock'
						$stock_sum = $this->variable_product_stock_sum( $id );

						if ( $stock_sum !== null ) {

							$product->set_manage_stock( true );

							$product->set_stock_quantity( $stock_sum );
							$product->save();
						}
					}
				}
			}
		}
	}


	/**
	 * @param int $product_id
	 *
	 * @return int|string
	 */
	public function variable_product_stock_sum( $product_id = 0 ) {
		global $wpdb;

		$product_id = intval( $product_id );

		// Get the product ID (can be defined)
		$product_id = $product_id > 0 ? $product_id : get_the_id();

		// Check and get the instance of the WC_Product Object
		$product = wc_get_product( $product_id );

		// Only for variable product type
		if ( $product && $product->is_type( 'variable' ) ) {

			// Get the stock quantity sum of all product variations (children).
			// When manage stock is no, stock quantity is NULL except when its manualy set via rest api.
			$stock_quantity = $wpdb->get_var( "SELECT SUM(mt1.meta_value)
				FROM {$wpdb->posts} as p 
				    INNER JOIN {$wpdb->postmeta} as mt1 ON ( p.ID = mt1.post_id )  
				    INNER JOIN {$wpdb->postmeta} AS mt2 ON ( p.ID = mt2.post_id )
					WHERE p.post_type = 'product_variation'
                    AND p.post_status = 'publish'
                AND p.post_parent = '{$product_id}'
                AND ( 
                    ( mt1.meta_key = '_stock' AND mt1.meta_value IS NOT NULL  ) 
                        AND 
                    ( mt2.meta_key = '_manage_stock' AND mt2.meta_value = 'yes' ) 
                    );
                    " );


			// if null, there is manage stock in any variation
			return $stock_quantity;
		}

		return null;
	}


	/**
	 * @param $terms
	 * @param $taxonomies
	 * @param $args
	 * @param $term_query
	 *
	 * @return mixed
	 */
	public function variation_filter( $terms, $taxonomies, $args, $term_query ) {

		$enabled_attributes = [];

		foreach ( $this->woo_settings as $key => $setting ) {
			if ( strpos( $key, 'pa_' ) === 0 ) {
				$enabled_attributes[] = $key;
			}
		}

		if ( ( is_product_category() || is_shop() ) && isset( $taxonomies[0] ) && count( $taxonomies ) == 1 && in_array( $taxonomies[0], $enabled_attributes ) ) {

			delete_transient( 'syncspider_filter_' . $taxonomies[0] );
			// todo: optimize queries
			// use transients, slow queries
			if ( false === ( $found_terms = get_transient( 'syncspider_filter_' . $taxonomies[0] ) ) ) {
				global $wpdb;
				$ext = "";
				$sp  = [];

				$simple_args = array(
					'post_type'      => 'product',
					'tax_query'      => array(
						array(
							'taxonomy' => $taxonomies[0],
							'operator' => 'EXISTS'
						),
						array(
							'taxonomy' => 'product_type',
							'field'    => 'slug',
							'terms'    => 'variable',
							'operator' => 'NOT IN',
						),
					),
					'fields'         => 'ids',
					'posts_per_page' => - 1
				);


				if ( is_product_category() ) {

					$simple_args['tax_query'][] = array(
						'taxonomy' => 'product_cat',
						'terms'    => get_queried_object_id(),
					);

					$args   = array(
						'post_type'      => 'product',
						'tax_query'      => array(
							array(
								'taxonomy' => 'product_cat',
								'terms'    => get_queried_object_id(),
							),
							array(
								'taxonomy' => 'product_type',
								'field'    => 'slug',
								'terms'    => 'variable',
							),
						),
						'fields'         => 'ids',
						'posts_per_page' => - 1
					);
					$query  = get_posts( $args );
					$in_ids = implode( ",", $query );

					if ( $in_ids ) {
						$ext = " and post_parent in ({$in_ids})";
					}
				}

				//all simple products that have selected taxonomy
				$simple_posts = get_posts( $simple_args );
				if ( $simple_posts ) {
					remove_filter( 'get_terms', array( $this, 'variation_filter' ), 10 );
					$sp = wp_get_object_terms( $simple_posts, $taxonomies[0], array( 'fields' => 'slugs' ) );
					add_filter( 'get_terms', array( $this, 'variation_filter' ), 10, 4 );
				}


				$prod_query = "
				SELECT distinct meta_value 
				FROM `{$wpdb->prefix}postmeta` 
				WHERE meta_key = 'attribute_{$taxonomies[0]}' 
				AND `post_id` in (select ID from `{$wpdb->prefix}posts` where post_type = 'product_variation' and post_status = 'publish' {$ext})";


				$all_publish = $wpdb->get_results( $prod_query, OBJECT_K );

				$all_publish = array_merge( array_keys( $all_publish ), $sp );

				foreach ( $terms as $key => $term ) {
					if ( ! in_array( $term->slug, $all_publish ) ) {
						unset( $terms[ $key ] );
					}
				}

				$found_terms = $terms;
				set_transient( 'syncspider_filter_' . $taxonomies[0], $found_terms, 1 * MINUTE_IN_SECONDS );

			}

			return $found_terms;
		}

		return $terms;
	}

	/**
	 * @param $item
	 * @param $cart_item_key
	 * @param $values
	 * @param $order
	 */
	function save_regular_line_item( $item, $cart_item_key, $values, $order ) {
		$product       = $values['data'];
		$regular_price = get_post_meta( $product->get_id(), '_regular_price', true );
		$item->add_meta_data( '_syncspider_original_price', $regular_price );
	}


	/**
	 * @param string $text
	 *
	 * @return array|string|string[]|null
	 */
	public function clean_text( string $text ) {
		$text = strip_shortcodes( $text );
		$text = preg_replace( "/\[(\/*)(.*?)\]/", " ", $text );

		return $text;
	}


	/**
	 * Check if option is set
	 *
	 * @param $option_name
	 *
	 * @return bool
	 */
	private function opt_is( $option_name ) {
		return isset( $this->woo_settings[ $option_name ] );
	}


	/**
	 * Return option value, false if option doesnt exist
	 *
	 * @param $option_name
	 *
	 * @return bool
	 */
	private function opt_val( $option_name ) {
		if ( isset( $this->woo_settings[ $option_name ] ) ) {
			return $this->woo_settings[ $option_name ];
		}

		return "";
	}

	/**
	 * Helper function to check if string ends with string
	 *
	 * @param $haystack
	 * @param $needle
	 *
	 * @return bool
	 */
	function ends_with( $haystack, $needle ) {
		$length = strlen( $needle );

		return $length > 0 ? mb_substr( $haystack, - $length ) === $needle : true;
	}


	public function register_new_post_status() {
		$this->additional_order_statuses = get_option( 'syncspider_order_statuses' );

		if ( $this->additional_order_statuses ) {
			foreach ( $this->additional_order_statuses as $status_key => $status_name ) {
				register_post_status( 'wc-' . $status_key, array(
					'label'                     => $status_name,
					'public'                    => true,
					'exclude_from_search'       => false,
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
				) );
			}
		}
	}

	function new_order_statuses( $order_statuses ) {
		if ( ! empty( $this->additional_order_statuses ) ) {
			foreach ( $this->additional_order_statuses as $status_key => $status_name ) {
				$order_statuses[ 'wc-' . $status_key ] = $status_name;
			}
		}

		return $order_statuses;
	}


	public function profile_fields( $user ) { ?>
        <h3><?php _e( 'Customer number', 'syncspider' ); ?></h3>
        <table class="form-table">
            <tr>
                <th><label for="sysp_customer_number"><?php _e( 'Customer number', 'syncspider' ); ?></label></th>
                <td>
                    <input type="text" name="sysp_customer_number" id="sysp_customer_number"
                           value="<?php echo esc_attr( get_user_meta( $user->ID, 'sysp_customer_number', true ) ); ?>"
                           class="regular-text"/><br/>
                </td>
            </tr>
        </table>
		<?php
	}

	public function save_profile_fields( $user_id ) {
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return false;
		}
		update_user_meta( $user_id, 'sysp_customer_number', sanitize_text_field( $_POST['sysp_customer_number'] ) );
	}

	function user_table( $column ) {
		$column['sysp_customer_number'] = __( 'Customer number', 'syncspider' );

		return $column;
	}

	function user_table_row( $val, $column_name, $user_id ) {
		if ( $column_name === 'sysp_customer_number' ) {
			return get_user_meta( $user_id, 'sysp_customer_number', true );
		}

		return $val;
	}

	public function user_meta( $meta ) {
		$meta[] = array(
			'slug'  => 'sysp_customer_number',
			'label' => __( 'Customer number', 'syncspider' ),
	    );

	    return $meta;
	}


	public function create_order( WC_Order $order, $data ) {
		$customer_id = $order->get_customer_id();

		if ( $customer_id ) {
			$customer_number = get_user_meta( $customer_id, 'sysp_customer_number', true );
			if ( $customer_number ) {
				$order->update_meta_data( 'sysp_customer_number', $customer_number );
			}
		}
	}


	public function display_order_meta( WC_Order $order ) {
		$customer_number = $order->get_meta( 'sysp_customer_number' );
		if ( $customer_number ) {
			?>
            <div class="order_data_column">
                <p><strong><?php _e( 'Customer number', 'syncspider' ) ?>:</strong> <?php echo $customer_number; ?></p>
            </div>
			<?php
		}
	}


}
