<?php

defined('ABSPATH') || exit;

if (!class_exists('WC_REST_Products_Controller')) {
	return;
}

/**
 * Controller for getting Woo Products info
 */
class SYSP_woo_varproducts extends WC_REST_Products_Controller
{
	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
//    protected $namespace = 'wp/v2/syncspider/products';


	/**
	 * Make post_type argument
	 *
	 * @param WP_REST_Request $request Request data.
	 *
	 * @return array
	 */
	protected function prepare_objects_query($request)
	{
		$args = parent::prepare_objects_query($request);
		$args = $this->syncspider_product_metadata($args, $request);

		return $args;
	}

	/**
	 *  check request for valid meta fields, add query args if necessary
	 *
	 * @param $args
	 * @param $request
	 * @return mixed
	 */
	public function syncspider_product_metadata($args, $request){

		$knownProductParams = [
			// oauth
			'oauth_consumer_key' => '',
			'oauth_signature_method' => '',
			'oauth_signature' => '',
			'oauth_timestamp' => '',
			'oauth_nonce' => '',
			'oauth_version' => '',

			// 14.11.2020 - https://woocommerce.github.io/woocommerce-rest-api-docs/#products
			'context' => '',
			'page' => '',
			'per_page' => '',
			'search' => '',
			'after' => '',
			'before' => '',
			'exclude' => '',
			'include' => '',
			'offset' => '',
			'order' => '',
			'orderby' => '',
			'parent' => '',
			'parent_exclude' => '',
			'slug' => '',
			'status' => '',
			'type' => '',
			'sku' => '',
			'featured' => '',
			'category' => '',
			'tag' => '',
			'shipping_class' => '',
			'attribute' => '',
			'attribute_term' => '',
			'tax_class' => '',
			'on_sale' => '',
			'min_price' => '',
			'max_price' => '',
			'stock_status' => '',
		];

		// ignoring documented parameters - just in case
		$validParams = array_diff_key( $request->get_query_params(), $knownProductParams );

		if ( isset( $validParams['sysp_author'] ) && is_numeric( $validParams['sysp_author'] ) ) {
			$args['author'] = $validParams['sysp_author'];
			unset( $validParams['sysp_author'] );
		}

		if ( isset( $validParams['sysp_created_after_gmt'] ) ) {

			$args['date_query'] = array(
				array(
					'column' => 'post_date_gmt',
					'after'  => $validParams['sysp_created_after_gmt'],
				),
			);
			unset( $validParams['sysp_created_after_gmt'] );
		}

		if ( isset( $validParams['sysp_modified_after_gmt'] ) ) {
			$args['date_query'] = array(
				array(
					'column' => 'post_modified_gmt',
					'after'  => $validParams['sysp_modified_after_gmt'],
				),
			);
			unset( $validParams['sysp_modified_after_gmt'] );
		}


		// get available product meta
		$productMeta = SYSP_rest::get_post_meta_fields('product');
		SYSP_rest::exclude_default_product_tags($productMeta);

		// additional syncpider meta fields
		$add_meta_fields = apply_filters( 'syncspider_add_product_meta', array() );

		if ( ! empty( $add_meta_fields ) ) {
			$add_meta_fields = wp_list_pluck( $add_meta_fields, 'label', 'slug' );
			$productMeta = $productMeta + $add_meta_fields;
		}
		$metaArgs = [];

		foreach ($validParams as $key => $param){
			if(array_key_exists($key, $productMeta)) {
				$metaArgs[] = [
					'key' => $key,
					'value' => $param,
				];
			}
		}

		if ( $metaArgs ) {
			$args['meta_query'] = $metaArgs;
			// also checking variations
			$args['post_type'] = array( 'product', 'product_variation' );
			// if we have more than 1, set reletion AND
			if ( count( $metaArgs ) > 1 ) {
				$args['meta_query']['relation'] = 'AND';
			}
		}

		return $args;
	}
}


