<?php

defined( 'ABSPATH' ) || exit;

class SYSP_Wp_Rest_Filter {

	private $version;

	/**
	 * SYSP_Wp_Rest_Filter constructor.
	 */
	public function __construct( $version ) {

		$this->version = $version;
		add_action( 'rest_api_init', array( $this, 'syncspider_rest_filter_add_filters' ) );

//		add_action( 'update_user_meta', array( $this, 'update_user_meta_timestamp' ), 10, 4 );
	}

	/**
	 * Add the necessary filter to each post type
	 **/
	public function syncspider_rest_filter_add_filters() {
		if ( SYSP_check::is_auth() ) {
			/* Adds Filter for All Post Types */
			foreach ( get_post_types( array( 'show_in_rest' => true ), 'objects' ) as $post_type ) {
				add_filter( 'rest_' . $post_type->name . '_query', array( $this, 'rest_filter_add_filter_param' ), 10, 2 );
			}

			/* Adds Filter for All Taxonomy Types */
			foreach ( get_taxonomies( array( 'show_in_rest' => true ), 'objects' ) as $tax_type ) {
				add_filter( 'rest_' . $tax_type->name . '_query', array( $this, 'rest_filter_add_filter_param' ), 10, 2 );
			}

			/* Adds Filter for User Type */
			add_filter( 'rest_user_query', array( $this, 'rest_filter_add_filter_param' ), 10, 2 );

			add_filter( 'rest_user_collection_params', array( $this, 'user_collection_params' ), 10, 1 );

			add_filter( 'rest_user_query', array( $this, 'user_modified_after' ), 10, 2 );

		}
	}


	/**
	 * Add the filter parameter
	 *
	 * @param array $args The query arguments.
	 * @param WP_REST_Request $request Full details about the request.
	 *
	 * @return array $args.
	 **/
	public function rest_filter_add_filter_param( $args, $request ) {

		// Bail out if no filter parameter is set.
		if ( empty( $request['filter'] ) || ! is_array( $request['filter'] ) ) {
			return $args;
		}

		$filter = $request['filter'];

		global $wp;

		$vars = apply_filters( 'rest_query_vars', $wp->public_query_vars );

		function allow_meta_query( $valid_vars ) {
			$valid_vars = array_merge( $valid_vars, array( 'meta_query', 'meta_key', 'meta_value', 'meta_compare' ) );

			return $valid_vars;
		}

		$vars = allow_meta_query( $vars );

		foreach ( $vars as $var ) {
			if ( isset( $filter[ $var ] ) ) {
				$args[ $var ] = $filter[ $var ];
			}
		}

		return $args;
	}

	/**
	 * Add 'registered_after' - date field in get rest query parameters.
	 *
	 * @param $query_params
	 *
	 * @return mixed
	 */
	function user_collection_params( $query_params ) {
		if ( ! isset( $query_params['registered_after'] ) ) {
			$query_params['registered_after'] = array(
				'description' => __( 'Limit response to users registered after a given ISO8601 compliant date. Syncspider field.', 'syncspider' ),
				'type'        => 'string',
				'format'      => 'date-time',
			);
		}

		if ( ! isset( $query_params['modified_after'] ) ) {
			$query_params['modified_after'] = array(
				'description' => __( 'Limit response to users modified after a given ISO8601 compliant date. Syncspider field.', 'syncspider' ),
				'type'        => 'string',
				'format'      => 'date-time',
			);
		}

		return $query_params;
	}

	/**
	 * Add query args if valid date is passed in parameter 'registered_after' or 'updated_after'.
	 *
	 * @param $args
	 * @param $request
	 *
	 * @return mixed
	 */
	public function user_modified_after( $args, $request ) {

		if ( isset( $request['registered_after'] ) ) {
			$args['date_query'] = array(
				array( 'after' => $request['registered_after'], 'inclusive' => false )
			);
		}

//		if ( isset( $request['modified_after'] ) ) {
//			$modified_timestmap = strtotime( $request['modified_after'] );
//
//			$args['meta_query'] = array(
//				array(
//					'key'     => 'synspider_user_updated',
//					'value'   => (string) $modified_timestmap,
//					'compare' => '>',
//				),
//			);
//
//		}

		return $args;
	}

	/**
	 * Hooks into the update_user_meta function.
	 * Set 'synspider_user_updated' to the current timestamp UTC.
	 *
	 * @param $meta_id
	 * @param $user_id
	 * @param $meta_key
	 * @param $_meta_value
	 */
	public function update_user_meta_timestamp( $meta_id, $user_id, $meta_key, $_meta_value ){
		remove_action( 'update_user_meta', array( $this, 'update_user_meta_timestamp' ), 10 );
		update_user_meta( $user_id, 'synspider_user_updated', gmdate( 'U' ) );
	}
}




