<?php


class SYSP_rest {

	public function __construct() {

		add_action( 'rest_api_init', array( $this, 'register_routes' ) );
		if ( function_exists( 'icl_object_id' ) ) {
			add_action( 'rest_api_init', array( $this, 'register_wpml_routes' ) );
		}
	}


	/**
	 *  Non-public and public routes
	 */
	public function register_routes() {

		/** Public */

		/*** Validate generated API key ***/
		register_rest_route( 'wp/v2/syncspider', 'validate', array(
				'methods'             => 'GET',
				'callback'            => array( $this, 'validate_key' ),
				'permission_callback' => '__return_true',
			)
		);


		/** non public */
		if ( SYSP_check::check_rest_view() ) {

			/*** https://example.com/wp-json/wp/v2/syncspider/metadata/?type=product ***/
			register_rest_route( 'wp/v2/syncspider', 'metadata', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_meta' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'metadata', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_meta' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);


			register_rest_route( 'wp/v2/syncspider', 'roles', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'user_roles'),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);


			/*** https://example.com/wp-json/wp/v2/syncspider/invoices/(?P<id>\d+) ***/
			register_rest_route( 'wp/v2/syncspider', 'invoices/(?P<id>\d+)', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'germanized_invoices' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'invoices/(?P<id>\d+)', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'germanized_invoices' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			/*** https://example.com/wp-json/wp/v2/syncspider/plugin/active/?plugin=woocommerce-germanized-pro ***/
			register_rest_route( 'wp/v2/syncspider', 'plugin/active', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'is_plugin_active' ),
					'permission_callback' => array( $this, 'permission_check' )
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'plugin/active', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'is_plugin_active' ),
					'permission_callback' => array( $this, 'permission_check' )
				)
			);


			register_rest_route( 'wp/v2/syncspider', 'tax', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_tax_rate' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);
			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'tax', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'get_tax_rate' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			/** ATUM routes */

			/*** https://example.com/wp-json/wp/v2/syncspider/inventory/(?P<id>\d+) ***/
			register_rest_route( 'wp/v2/syncspider', 'inventory/(?P<id>\d+)', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'atum_inventory_get' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'inventory/(?P<id>\d+)', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'atum_inventory_get' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);


			/*** https://example.com/wp-json/wp/v2/syncspider/inventory/(?P<id>\d+) ***/
			register_rest_route( 'wp/v2/syncspider', 'inventory', array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'atum_inventory_post' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'inventory', array(
					'methods'             => 'POST',
					'callback'            => array( $this, 'atum_inventory_post' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			if ( class_exists( 'WCFMmp_Dependencies' ) ) {
				register_rest_route( 'wp/v2/syncspider', 'vendors', array(
						'methods'             => 'GET',
						'callback'            => array( $this, 'wc_vendors' ),
						'permission_callback' => array( $this, 'permission_check' ),
					)
				);
				// mistake, will be removed in the future
				register_rest_route( 'wp/v3/syncspider', 'vendors', array(
						'methods'             => 'GET',
						'callback'            => array( $this, 'wc_vendors' ),
						'permission_callback' => array( $this, 'permission_check' ),
					)
				);
			}
		}
	}


	/**
	 * Register wpml routes
	 */
	public function register_wpml_routes() {
		if ( SYSP_check::check_rest_view() ) {
			register_rest_route( 'wp/v2/syncspider', 'wpml', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'syncspider_wpml' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);

			// mistake, will be removed in the future
			register_rest_route( 'wp/v3/syncspider', 'wpml', array(
					'methods'             => 'GET',
					'callback'            => array( $this, 'syncspider_wpml' ),
					'permission_callback' => array( $this, 'permission_check' ),
				)
			);
		}
	}


	/**
	 * Check permissions. Our routes are not public so this might not be needed.
	 *
	 * @return bool
	 */
	public function permission_check() {
		if ( SYSP_check::check_rest_view() ) {
			return true;
		}

		return false;
	}

	/**
	 * Check if api key is valid.
	 *
	 * @return bool|WP_Error
	 */
	public function validate_key() {

		$sysp_hash = get_option( 'wp_syncspider_api_hash' );
		if ( ! $sysp_hash ) {
			return new WP_Error( 'missing_api_key', __( 'There is no API key generated in database', 'syncspider' ), array( 'status' => 404 ) );
		}
		if ( isset( $_GET['api_key'] ) ) {
			$token = $_GET['api_key'];
			if ( ! empty( $token ) && wp_check_password( $token, $sysp_hash ) ) {
				return true;
			}
		} else {
			return new WP_Error( 'missing_api_key', __( 'API key is required in query params', 'syncspider' ), array( 'status' => 404 ) );
		}

		return new WP_Error( 'invalid_api_key', __( 'Invalid API key', 'syncspider' ), array( 'status' => 404 ) );
	}


	/**
	 * Germanized invoices - adapted to HPOS
	 */
	public function germanized_invoices( WP_REST_Request $request ) {

		$orderId    = $request->get_param( 'id' );
		$invoices = [];

		if ( function_exists( 'sab_get_invoices' ) ) {
			$sab_invoices = sab_get_invoices( array( 'reference_id' => $orderId ) );

			if ( ! empty( $sab_invoices ) && is_array( $sab_invoices ) ) {
				foreach ( $sab_invoices as $invoice ) {
					$invoice_total = $invoice->get_total();
					$invoices[]    = [
						'invoice_number' => $invoice->get_formatted_number(),
						'invoice_total'  => $invoice_total ?? null
					];
				}
			}
		}
		return $invoices;
	}


	/**
	 * Check if given plugin is installed and active.
	 *
	 * @return bool
	 */
	public function is_plugin_active() {

		if ( isset( $_GET['plugin'] ) ) {
			$plugin_name   = $_GET['plugin'];
			$plugin_folder = $_GET['plugin'];
		}

		if ( isset( $_GET['pluginFolder'] ) ) {
			$plugin_folder = $_GET['pluginFolder'];
		}

		if ( $plugin_name ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			if ( is_plugin_active( $plugin_folder . '/' . $plugin_name . '.php' ) ) {
				return true;
			}
		}

		return false;
	}


	/**
	 * Get tax rate
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return int|mixed
	 */
	public function get_tax_rate( WP_REST_Request $request ) {

		$data = $request->get_params();

		if ( isset( $data['base'], $data['tax_class'] ) && $data['base'] == true ) {
			$rates = WC_Tax::get_base_tax_rates( $data['tax_class'] );
			if ( ! empty( $rates ) ) {
				$rate = reset( $rates );

				return $rate['rate'];
			}
		}

		if ( isset( $data['country'], $data['state'], $data['city'], $data['postcode'], $data['tax_class'] ) ) {
			$rates = WC_Tax::find_rates( $data );
			if ( ! empty( $rates ) ) {
				$rate = reset( $rates );

				return $rate['rate'];
			}
		}

		return 0;
	}


	/**
	 * Get inventory info
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return array|object|null
	 */
	public function atum_inventory_get( WP_REST_Request $request ) {
		global $wpdb;

		$productId = $request->get_param( 'id' );

		$inventories = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT inv.id, name, product_id, priority, region, inventory_date, bbe_date, is_main, lot, write_off,
                    inbound_stock, stock_on_hold, sold_today, sales_last_days, reserved_stock, customer_returns,
                    warehouse_damage, lost_in_post, other_logs, out_stock_days, lost_sales, update_date,
                    meta.id as meta_id, meta.inventory_id, sku, manage_stock, stock_quantity, backorders,
                    stock_status, supplier_id, supplier_sku, sold_individually, out_stock_threshold, purchase_price,
                    price, regular_price, sale_price, date_on_sale_from, date_on_sale_to, out_stock_date,
                    regions FROM {$wpdb->prefix}atum_inventories inv
                    LEFT JOIN {$wpdb->prefix}atum_inventory_meta meta ON inv.id = meta.inventory_id
                    LEFT JOIN (SELECT atumz.inventory_id as zone_inv_id, GROUP_CONCAT(woozone.zone_name) as regions
                    FROM {$wpdb->prefix}atum_inventory_regions atumz LEFT JOIN {$wpdb->prefix}woocommerce_shipping_zones woozone
                    ON atumz.zone_id = woozone.zone_id group by atumz.inventory_id) zon
                    ON inv.id = zone_inv_id
                    WHERE product_id = {$productId}
                    ORDER BY inv.id"
			)
		);

		if ( $inventories ) {
			// fetch location
			foreach ( $inventories as &$inventory ) {
				$inventory->location = null;

				if ( $inventory->id ) {
					$result = $wpdb->get_results(
						$wpdb->prepare( "SELECT term_id FROM {$wpdb->prefix}atum_inventory_locations inv
					LEFT JOIN {$wpdb->prefix}term_taxonomy tax ON inv.term_taxonomy_id = tax.term_taxonomy_id
					WHERE inv.inventory_id = {$inventory->id}
					ORDER BY inv.inventory_id;"
						) );

					if ( isset( $result[0] ) && $result[0]->term_id ) {
						$location = $wpdb->get_results(
							$wpdb->prepare( "SELECT name FROM {$wpdb->prefix}terms where term_id = {$result[0]->term_id}"
							) );
						if ( isset( $location[0] ) && $location[0]->name ) {
							$inventory->location = $location[0]->name;
						}
					}

					// add supplier name for inventory
					if ( $inventory->is_main ) {
						$result = $wpdb->get_results(
							$wpdb->prepare( "SELECT * FROM {$wpdb->prefix}atum_product_data where product_id = {$inventory->product_id}"
							) );
						if ( isset( $result[0] ) && $result[0]->supplier_id ) {
							$post                     = get_post( $result[0]->supplier_id );
							$inventory->supplier_name = $post->post_title;
							if ( isset( $result[0] ) && $result[0]->supplier_sku ) {
								$inventory->supplier_sku = $result[0]->supplier_sku;
							}
						}
					} else {
						if ( $inventory->supplier_id ) {
							$post                     = get_post( $inventory->supplier_id );
							$inventory->supplier_name = $post->post_title;
						}
					}
				}
			}
		}

		return $inventories;
	}

	/**
	 * Create / Update inventory
	 *
	 * @param WP_REST_Request $request
	 *
	 * @return WP_REST_Response
	 */
	public function atum_inventory_post( WP_REST_Request $request ) {
		global $wpdb;

		$data = $request->get_json_params();

		if ( ! isset( $data['name'], $data['sku'], $data['stock_quantity'] ) ) {
			$message = '';
			if ( ! isset( $data['sku'] ) ) {
				$message = 'Missing SKU for inventory data ' . $data['name'] ?? '';
			}
			if ( ! isset( $data['name'] ) ) {
				$message = 'Missing name for inventory data with Product SKU: ' . $data['sku'] ?? '';
			}
			if ( ! isset( $data['stock_quantity'] ) ) {
				$message = 'Missing stock_quantity for inventory data: ' . $data['name'] ?? '' . ' Product SKU: ' . $data['sku'] ?? '';
			}

			return new WP_REST_Response( [
				'code'    => 400,
				'status'  => 'error',
				'message' => "Missing inventory data! " . $message
			], 400 );
		}

		$productId = wc_get_product_id_by_sku( $data['sku'] );

		if ( ! $productId ) {
			return new WP_REST_Response( [
				'code'    => 400,
				'status'  => 'error',
				'message' => "Product not found!" . "SKU: " . $data['sku'] ?? ''
			], 400 );
		}

		$name     = $data['name'];
		$quantity = (int) $data['stock_quantity'];

		if ( $quantity > 0 ) {
			$stock_status = 'instock';
		} else {
			$stock_status = 'outofstock';
		}

		// fist check if inventory exists
		$result = $wpdb->get_results( $wpdb->prepare( "SELECT id FROM {$wpdb->prefix}atum_inventories inv where inv.product_id = {$productId} AND inv.name = '{$name}'" ) );

		if ( isset( $result[0] ) && $result[0]->id ) {

			// fetch inventory meta for specific inventory
			$inventory_result = $wpdb->get_results( $wpdb->prepare( "SELECT id FROM {$wpdb->prefix}atum_inventory_meta where inventory_id = {$result[0]->id}" ) );

			if ( isset( $inventory_result[0] ) && $inventory_result[0]->id ) {
				// update quantity
				$wpdb->get_results( $wpdb->prepare( "UPDATE {$wpdb->prefix}atum_inventory_meta SET stock_quantity = {$quantity},
			stock_status = '{$stock_status}' where id = {$inventory_result[0]->id}" ) );
			}

			return new WP_REST_Response( [
				'code'    => 200,
				'status'  => 'updated',
				'message' => "Successfully updated Inventory with ID: " . $result[0]->id
			], 200 );
		}

		// create new inventory
		$wpdb->insert( $wpdb->prefix . 'atum_inventories', array(
			'product_id' => $productId,
			'name'       => $name,
		) );

		$lastid = $wpdb->insert_id;

		// create new inventory meta
		$wpdb->insert( $wpdb->prefix . 'atum_inventory_meta', array(
			'inventory_id'   => $lastid,
			'stock_quantity' => $quantity,
			'manage_stock'   => 1,
			'stock_status'   => $stock_status,
			'sku'            => $data['sku']
		) );

		return new WP_REST_Response( [
			'code'    => 200,
			'status'  => 'created',
			'message' => 'Successfully created Inventory with ID: ' . $lastid
		], 200 );
	}


	function wc_vendors() {

		$args = array(
			'role'  => 'wcfm_vendor',
			'order' => 'ASC'
		);

		return get_users( $args );
	}

	public function get_meta() {

		 if ( isset( $_GET['type'] ) && ( isset( get_post_types()[ $_GET['type'] ] ) || $_GET['type'] == 'user' ) ) {
			$post_type = $_GET['type'];
		} elseif(isset( $_GET['type']) &&  $_GET['type'] == 'order') {
			$post_type = 'order';
		} else {
			$post_type = 'post';
		}

		switch ( $post_type ) {
			case 'user':
				$meta_values = $this->get_user_meta_fields();
				if ( empty( $meta_values ) ) {
					return new WP_REST_Response( __( 'Auth token missing or user has no meta data.', 'syncspider' ), 404 );
				}
				break;
			case 'order':
				return $this->get_order_meta_fields();
				break;
			case 'product':
				$meta_values = $this->get_post_meta_fields( $post_type );
				$this->exclude_default_product_tags( $meta_values );
				if ( class_exists( 'WCFMmp_Dependencies' ) ) {
					$meta_values['vendor_name'] = 1;
				}
				break;
			default:
				$meta_values = $this->get_post_meta_fields( $post_type );
				break;
		}

		$syncoutput = array();

		if ( $meta_values ) {
			foreach ( $meta_values as $value => $data ) {
				// check if string starts with _, if it does check if its duplicate (for ACF double field bug)
				if ( '_' === $value[0] ) {
					if ( isset( $meta_values[ substr( $value, 1 ) ] ) ) {
						continue;
					}
				}
				// add slug / label
				// Label is same as slug
				$syncoutput[] = array(
					'slug'  => $value,
					'label' => $value,
				);
			}
		}

		return apply_filters( 'syncspider_add_' . $post_type . '_meta', $syncoutput );
	}



	public static function get_post_meta_fields( $postType ) {

		$meta = [];

		$args = array(
			'post_type'   => $postType,
			'numberposts' => 1,
			'orderby'     => 'date',
			'post_status' => array( 'publish', 'pending', 'draft', 'private', 'trash' ),
			'order'       => 'ASC'
		);

		$firstPost = get_posts( $args );

		if ( $firstPost ) {
			$meta = get_post_custom( $firstPost[0]->ID );
		}

		return $meta;
	}

	private function get_user_meta_fields() {
		if ( SYSP_check::is_auth() ) {
			$current_user = wp_get_current_user();

			return get_metadata( 'user', $current_user->ID );
		} else {
			return false;
		}
	}

	/**
	 *  Get order meta fields - adapted for HPOS
	 */
	public static function get_order_meta_fields( $retArr = false ) {
		$order_meta = [];

		$args = array(
			'limit'   => 1,
			'orderby' => 'date',
			'order'   => 'ASC',
		);

		$oldest_order = wc_get_orders( $args );

		if ( empty( $oldest_order ) ) {
			return $order_meta;
		}

		$oldest_order = reset( $oldest_order );

		// get meta keys for first order
		$order_meta_keys = wp_list_pluck( $oldest_order->get_meta_data(), 'key' );

		// TODO separate these default order tags
		$default_woo_order_meta_tags = array(
			"_order_key",
			"_customer_user",
			"_payment_method",
			"_payment_method_title",
			"_customer_user_agent",
			"_created_via",
			"_cart_hash",
			"_billing_first_name",
			"_billing_last_name",
			"_billing_address_1",
			"_billing_city",
			"_billing_postcode",
			"_billing_country",
			"_billing_email",
			"_billing_phone",
			"_order_currency",
			"_cart_discount",
			"_cart_discount_tax",
			"_order_shipping",
			"_order_shipping_tax",
			"_order_tax",
			"_order_total",
			"_order_version",
			"_prices_include_tax",
			"_billing_address_index",
			"_shipping_address_index",
			"is_vat_exempt",
			"_recorded_sales",
			"_recorded_coupon_usage_counts",
			"_order_stock_reduced",
			"_edit_lock",
			"_edit_last",
			"_date_completed",
			"_date_paid",
			"_paid_date",
			"_completed_date",
			"_download_permissions_granted",
			"_shipping_first_name",
			"_shipping_last_name",
			"_shipping_address_1",
			"_shipping_city",
			"_shipping_postcode",
			"_shipping_country",
			"_shipping_email",
			"_shipping_phone",
			"_billing_lat",
			"_shipping_lat",
			"_billing_lng",
			"_shipping_lng",
			"_billing_street_number",
			"_shipping_street_number"
		);

		if ( $order_meta_keys ) {
			// subtract not needed meta keys
			$order_meta_keys = array_diff( $order_meta_keys, $default_woo_order_meta_tags );

			foreach ( $order_meta_keys as $key => $value ) {
				// check if string starts with _, if it does check if its duplicate (for ACF double bug)
				if ( '_' === $value[0] ) {
					if ( false !== array_search( substr( $value, 1 ), $order_meta_keys, true ) ) {
						continue;
					}
				}
				// add slug / label
				// Label is same as slug
				$order_meta[] = array(
					'slug'  => $value,
					'label' => $value,
				);
			}
		}

		$order_meta = apply_filters( 'syncspider_add_order_meta', $order_meta );

		return ! $retArr ? rest_ensure_response( $order_meta ) : $order_meta;
	}


	public static function exclude_default_product_tags( &$meta ) {

		foreach ( $meta as $key => $meta_field ) {
			//exclude these from meta fields
			$defaultwoometatags = array(
				"_wc_review_count",
				"_wc_rating_count",
				"_wc_average_rating",
				"_edit_lock",
				"_edit_last",
				"_sku",
				"_sale_price_dates_from",
				"_sale_price_dates_to",
				"total_sales",
				"_tax_status",
				"_tax_class",
				"_manage_stock",
				"_backorders",
				"_sold_individually",
				"_weight",
				"_length",
				"_width",
				"_height",
				"_upsell_ids",
				"_crosssell_ids",
				"_purchase_note",
				"_default_attributes",
				"_virtual",
				"_downloadable",
				"_product_image_gallery",
				"_download_limit",
				"_download_expiry",
				"_stock",
				"_stock_status",
				"_product_attributes",
				"_product_version",
				"_regular_price",
				"_sale_price",
				"_et_pb_post_hide_nav",
				"_et_pb_page_layout",
				"_et_pb_side_nav",
				"_thumbnail_id",
				"_price",
				"_et_pb_use_builder",
				"_et_builder_version",
				"_et_pb_ab_stats_refresh_interval",
				"_et_pb_old_content",
				"_et_pb_enable_shortcode_tracking",
				"_et_pb_custom_css",
				"et_enqueued_post_fonts",
				"_digital",
				"_downloadable_files",
				"_wpml_word_count",
				"_wpml_media_featured",
				"_wpml_media_duplicate",
				"_wpml_location_migration_done",
				"_sale_label",
				"_low_stock_amount",
				"_et_pb_show_page_creation",
				"_wp_old_slug"
			);
			if ( in_array( $key, $defaultwoometatags ) ) {
				unset( $meta[ $key ] );
			}

			//exclude B2B Market plugin price, price_type fields
			if ( preg_match( '/\bbm_([^_\n]+)*_*\d([^_]+)*_price|_type$/', $key ) ) {
				unset( $meta[ $key ] );
			}
		}
	}


	public function user_roles() {
		global $wp_roles;

		$roles = $wp_roles->roles;

		foreach ( $roles as $roleKey => &$role ) {
			$role['slug'] = $roleKey;
		}

		$roles['guest'] = [
			'name' => 'Guest',
			'slug' => 'guest'
		];

		return $roles;
	}

	public function syncspider_wpml() {
		global $wpdb;
		$lang      = 'en';
		$response  = array();
		$languages = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT code, english_name, active, tag, name, default_locale
            FROM {$wpdb->prefix}icl_languages lang
            INNER JOIN {$wpdb->prefix}icl_languages_translations trans
            ON lang.code = trans.language_code
            AND trans.display_language_code=%s"
				,
				$lang
			)
		);

		$my_default_lang = apply_filters( 'wpml_default_language', null );

		foreach ( $languages as $l ) {
			if ( $l->active == 1 ) {
				$response[] = array(
					'name'    => $l->name,
					'code'    => $l->tag,
					'default' => $l->tag === $my_default_lang,
					'locale'  => $l->default_locale,
				);
			}
		}

		return rest_ensure_response( $response );
	}

}