<?php
/*
Plugin Name: Monobunt Syncspider
Plugin URI:  https://syncspider.com/
Description: Custom plugin for Syncspider
Version:     1.4.61
Author:      Monobunt
Author URI:  https://www.monobunt.at
License:     GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

define( 'SYNCSPIDER_VERSION', '1.4.61' );
define( 'SYNCSPIDER_PLUGIN_FILE', __FILE__ );

define( 'SYNCSPIDER_PLUGIN_URL', plugin_dir_url( __FILE__ ) );

define( 'SYNCSPIDER_ADDONS_DIR', dirname( SYNCSPIDER_PLUGIN_FILE ) . DIRECTORY_SEPARATOR . 'addons' );

require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-check.php';
require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-auth.php';
new SYSP_auth();


/**
 * Check if api key exists and create hash.
 */
register_activation_hook( SYNCSPIDER_PLUGIN_FILE, 'syncspider_register_plugin_settings' );
function syncspider_register_plugin_settings() {
	if ( get_option( 'wp_syncspider_api_key' ) ) {
		$password = get_option( 'wp_syncspider_api_key' );
		$hash     = wp_hash_password( $password );

		update_option( 'wp_syncspider_api_hash', $hash, true );
		update_option( 'wp_syncspider_user_id', get_current_user_id(), true );
		update_option( 'wp_syncspider_api_string', mb_substr( $password, 0, 5 ), true );

		delete_option( 'wp_syncspider_api_key' );
	}
}


require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-load-addons.php';
new SYSP_Load_Addons();

/**
 * Admin
 */

require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'class-sysp-admin.php';
new SYSP_Admin( SYNCSPIDER_VERSION );


// check if woo is active and rest is woo
if ( SYSP_check::check_woo() ) {
	// Woo settings
	$woo_se = get_option( 'syncspider_settings_woo' );

	// woo admin
	require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'admin' . DIRECTORY_SEPARATOR . 'class-sysp-woo-admin.php';
	new SYSP_Woo_Admin( SYNCSPIDER_VERSION );

	require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-woo-addons.php';
	new SYSP_Woo_Addons();

	if ( isset( $woo_se['image_checking'] ) ) {
		require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-woo-image-check.php';
		new SYSP_Image_Check( SYNCSPIDER_VERSION );
	}
} else {
	// remove Woocommerce from menu
	add_filter( 'syncspider_menu_items', function ( $menu_items ) {
		unset( $menu_items[100], $menu_items[1000] );

		return $menu_items;
	} );
}


/**
 * Rest api filters
 */
require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-wp-rest-filter.php';
new SYSP_Wp_Rest_Filter( SYNCSPIDER_VERSION );


// routes
require plugin_dir_path( SYNCSPIDER_PLUGIN_FILE ) . 'includes' . DIRECTORY_SEPARATOR . 'rest-api' . DIRECTORY_SEPARATOR . 'class-sysp-rest.php';
new SYSP_rest();


// Custom post types and users, include meta fields in response
add_action( 'rest_api_init', 'syncspider_register_meta_fields', 20 );
function syncspider_register_meta_fields() {

	// auth
	if ( SYSP_check::is_auth() ) {

		$meta_fields = get_metadata( 'user', get_option( 'wp_syncspider_user_id' ) );

		$meta_args = array(
			// Show in the WP REST API response. Default: false.
			'show_in_rest' => true,
		);
		foreach ( $meta_fields as $meta_field => $value ) {
			register_meta( 'user', $meta_field, $meta_args );
		}

		$post_types = get_post_types( [], 'names' );

		$exclude_types = [
			'product',
			'revision',
			'nav_menu_item',
			'custom_css',
			'customize_changeset',
			'oembed_cache',
			'user_request',
			'scheduled-action',
			'acf-field-group',
			'product_variation',
			'shop_order',
			'shop_order_refund',
			'shop_coupon',
			'customer_groups',
			'wp_block',
			'wp_block',
			'wp_template',
			'wp_template_part',
			'_wp_page_template',
			'wp_global_styles',
			'wp_navigation',
		];

		foreach ( $post_types as $post_type => $value ) {

			if ( in_array( $post_type, $exclude_types, true ) ) {
				continue;
			}

			$meta_fields = SYSP_rest::get_post_meta_fields( $post_type );

			// additional syncpider meta fields
			$add_meta_fields = apply_filters( 'syncspider_add_' . $post_type . '_meta', array() );

			if ( ! empty( $add_meta_fields ) ) {

				$add_meta_fields = wp_list_pluck( $add_meta_fields, 'label', 'slug' );

				$meta_fields = $meta_fields + $add_meta_fields;
			}


			if ( ! empty( $meta_fields ) ) {

				$registered_meta_fields = get_registered_meta_keys( 'post', $post_type );

				foreach ( $meta_fields as $meta_field => $val ) {

					$exists = $registered_meta_fields[ $meta_field ] ?? null;

					if ( $exists && $exists['type'] !== 'string' ) {
						continue;
					}

					$meta_args = array(
						// Show in the WP REST API response. Default: false.
						'show_in_rest'   => true,
						'object_subtype' => $post_type,
						'type'           => 'string',
						'single'         => true,
					);

					if ( ! is_array( $meta_field ) ) {
						register_meta( 'post', $meta_field, $meta_args );
					}
				}
			}
		}
	}
}


/**
 * Hook into order API response to add custom field data.
 *
 * @param $order_data   array
 * @param $order        WC_Order
 * @param $fields    array
 * @param $server    array
 *
 * @return $order_data    array
 */
add_filter( 'woocommerce_rest_prepare_shop_order_object', 'syncspider_order_api_response', 10, 3 );
function syncspider_order_api_response( $response, $order, $request ) {

	$user_info = get_userdata( $order->get_customer_id() );
	if ( $user_info ) {
		$customer_username = $user_info->user_login;
	} else {
		$customer_username = null;
	}
	$response->data['customer_username'] = $customer_username;

	return $response;
}


/**
 * Hook into product API response to add custom field data.
 *
 * @param $order_data   array
 * @param $product      WC_Product
 * @param $fields    array
 * @param $server    array
 *
 * @return $order_data    array
 */
add_filter( 'woocommerce_rest_prepare_product_object', 'syncspider_product_api_response', 10, 3 );
add_filter( 'woocommerce_rest_prepare_product_variation_object', 'syncspider_product_api_response', 10, 3 );
function syncspider_product_api_response( $response, $product, $request ) {

	if ( class_exists( 'WCFMmp_Dependencies' ) ) {
		$vendor_name                   = get_user_meta( get_post_field( 'post_author', $product->get_id() ), 'store_name', true );
		$response->data['vendor_name'] = $vendor_name ?: "";
	}

	return $response;
}


add_filter( 'woocommerce_rest_shop_order_object_query', 'syncspider_order_metadata', 15, 2 );
function syncspider_order_metadata( $args, $request ) {

	$knownOrderParams = [
		// oauth
		'oauth_consumer_key'     => '',
		'oauth_signature_method' => '',
		'oauth_signature'        => '',
		'oauth_timestamp'        => '',
		'oauth_nonce'            => '',
		'oauth_version'          => '',

		// 14.11.2020 - https://woocommerce.github.io/woocommerce-rest-api-docs/#list-all-orders
		'context'                => '',
		'page'                   => '',
		'per_page'               => '',
		'search'                 => '',
		'after'                  => '',
		'before'                 => '',
		'exclude'                => '',
		'include'                => '',
		'offset'                 => '',
		'order'                  => '',
		'orderby'                => '',
		'parent'                 => '',
		'parent_exclude'         => '',
		'status'                 => '',
		'customer'               => '',
		'product'                => '',
		'dp'                     => '',

	];

	// ignoring documented parameters - just in case
	$validParams = array_diff_key( $request->get_query_params(), $knownOrderParams );


	if ( isset( $validParams['author'] ) && is_numeric( $validParams['author'] ) ) {
		$args['author'] = $validParams['author'];
		unset( $validParams['author'] );
	}


	if ( isset( $validParams['sysp_created_after_gmt'] ) ) {
		$args['date_query'] = array(
			array(
				'column' => 'post_date_gmt',
				'after'  => $validParams['sysp_created_after_gmt'],
			),
		);
		unset( $validParams['sysp_created_after_gmt'] );
	}

	if ( isset( $validParams['sysp_modified_after_gmt'] ) ) {
		$args['date_query'] = array(
			array(
				'column' => 'post_modified_gmt',
				'after'  => $validParams['sysp_modified_after_gmt'],
			),
		);
		unset( $validParams['sysp_modified_after_gmt'] );
	}


	// get available order meta
	$order_meta = apply_filters( 'syncspider_allowed_rest_order_meta_params', SYSP_rest::get_order_meta_fields( true ) );

	$metaArgs = [];

	foreach ( $validParams as $key => $param ) {
		if ( in_array( $key, array_column( $order_meta, 'slug' ) ) ) {
			$metaArgs[] = [
				'key'   => $key,
				'value' => $param,
			];
		}
	}

	if ( $metaArgs ) {
		$args['meta_query'] = $metaArgs;
		// if we have more than 1, set relation AND
		if ( count( $metaArgs ) > 1 ) {
			$args['meta_query']['relation'] = 'AND';
		}
	}

	return $args;
}


add_filter( 'syncspider_allowed_rest_order_meta_params', 'syncspider_hc_allowed_rest_order_meta_params' );
function syncspider_hc_allowed_rest_order_meta_params( $meta ) {
	$meta[] = array(
		'slug'  => '_billing_country',
		'label' => '_billing_country'
	);

	$meta[] = array(
		'slug'  => '_shipping_country',
		'label' => '_shipping_country'
	);

	return $meta;
}


add_action( 'rest_api_init', 'syncspider_register_varproducts_api' );
function syncspider_register_varproducts_api() {
	// re register products routes to add query argument for meta data
	if ( class_exists( 'WC_REST_Products_Controller' ) ) {
		require plugin_dir_path( __FILE__ ) . 'includes' . DIRECTORY_SEPARATOR . 'class-sysp-woo-varproducts.php';
		$controller = new SYSP_woo_varproducts();
		$controller->register_routes();
	}
}


// set post author ID
add_action( 'woocommerce_rest_insert_product_object', 'syncspider_set_store_owner', 25, 3 );
add_action( 'woocommerce_rest_insert_product_variation_object', 'syncspider_set_store_owner', 25, 3 );
function syncspider_set_store_owner( $object, $request, $creating ) {
	if ( $object->get_meta( 'store_owner_id' ) ) {
		$post_author = (int) $object->get_meta( 'store_owner_id' );

		if ( get_user_by( 'id', $post_author ) ) {
			if ( apply_filters( 'syncspider_directly_set_store_owner_id', false, $object, $creating ) ) {
				syncspider_directly_assign_author( $object->get_id(), $post_author );
				syncspider_delete_store_owner_id( $object->get_id() );
			} else {
				$arg = array( 'ID' => $object->get_id(), 'post_author' => $object->get_meta( 'store_owner_id' ) );
				wp_update_post( $arg );
				delete_post_meta( $object->get_id(), 'store_owner_id' );
			}
		}

	}
}

/**
 * Assign author directly in db, avoid insert post hooks
 *
 * @param int $post_id
 * @param int $author_id
 *
 * @return bool|int|mysqli_result|null
 */
function syncspider_directly_assign_author( int $post_id, int $author_id ) {
	global $wpdb;

	return $wpdb->update( $wpdb->posts, array( 'post_author' => $author_id ), array( 'ID' => $post_id ) );
}

/**
 * Delete store_owner_id  directly in db, avoid delete_postmeta hooks
 *
 * @param int $post_id
 *
 * @return bool|int|mysqli_result|null
 */
function syncspider_delete_store_owner_id( int $post_id ) {
	global $wpdb;

	return $wpdb->delete( $wpdb->postmeta, array( 'post_id' => $post_id, 'meta_key' => 'store_owner_id' ) );
}


/**
 * Add default SyncSpider order meta
 */
add_filter( 'syncspider_add_order_meta', function ( $order_meta ) {

	$order_meta[] = array(
		'slug'  => "_syncspider_order_source",
		'label' => "SyncSpider Order Source",
	);

	$order_meta[] = array(
		'slug'  => "_dokan_vendor_id",
		'label' => "_dokan_vendor_id",
	);

	return $order_meta;
} );